@@ -17,6 +17,8 @@
 #include "math_.h"
 #include "memory_.h"
 #include "gx.h"
+#include "gsstruct.h"
+#include "gxobj.h"
 #include "gserrors.h"
 #include "gsropt.h"
 #include "gxcomp.h"
@@ -24,7 +26,8 @@
 #include "gxdevsop.h"
 #include "gdevp14.h"        /* Needed to patch up the procs after compositor creation */
 #include "gstrans.h"        /* For gs_pdf14trans_t */
-#include "gxistate.h"       /* for gs_image_state_s */
+#include "gxgstate.h"       /* for gs_image_state_s */
+
 
 /* defined in gsdpram.c */
 int gx_default_get_param(gx_device *dev, char *Param, void *list);
@@ -673,7 +676,10 @@ gx_device_fill_in_procs(register gx_device * dev)
     fill_dev_proc(dev, update_spot_equivalent_colors, gx_default_update_spot_equivalent_colors);
     fill_dev_proc(dev, ret_devn_params, gx_default_ret_devn_params);
     fill_dev_proc(dev, fillpage, gx_default_fillpage);
-    fill_dev_proc(dev, copy_alpha_hl_color, gx_default_copy_alpha_hl_color);
+    if ((dev_proc(dev, copy_planes) != NULL) &&
+        (dev_proc(dev, get_bits_rectangle) != NULL)) {
+        fill_dev_proc(dev, copy_alpha_hl_color, gx_default_copy_alpha_hl_color);
+    }
 
     /* NOT push_transparency_state */
     /* NOT pop_transparency_state */
@@ -832,7 +838,7 @@ gx_get_largest_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
 int
 gx_no_create_compositor(gx_device * dev, gx_device ** pcdev,
                         const gs_composite_t * pcte,
-                        gs_imager_state * pis, gs_memory_t * memory,
+                        gs_gstate * pgs, gs_memory_t * memory,
                         gx_device *cdev)
 {
     return_error(gs_error_unknownerror);	/* not implemented */
@@ -840,16 +846,16 @@ gx_no_create_compositor(gx_device * dev, gx_device ** pcdev,
 int
 gx_default_create_compositor(gx_device * dev, gx_device ** pcdev,
                              const gs_composite_t * pcte,
-                             gs_imager_state * pis, gs_memory_t * memory,
+                             gs_gstate * pgs, gs_memory_t * memory,
                              gx_device *cdev)
 {
     return pcte->type->procs.create_default_compositor
-        (pcte, pcdev, dev, pis, memory);
+        (pcte, pcdev, dev, pgs, memory);
 }
 int
 gx_null_create_compositor(gx_device * dev, gx_device ** pcdev,
                           const gs_composite_t * pcte,
-                          gs_imager_state * pis, gs_memory_t * memory,
+                          gs_gstate * pgs, gs_memory_t * memory,
                           gx_device *cdev)
 {
     *pcdev = dev;
@@ -860,7 +866,7 @@ gx_null_create_compositor(gx_device * dev, gx_device ** pcdev,
  * Default handler for creating a compositor device when writing the clist. */
 int
 gx_default_composite_clist_write_update(const gs_composite_t *pcte, gx_device * dev,
-                gx_device ** pcdev, gs_imager_state * pis, gs_memory_t * mem)
+                gx_device ** pcdev, gs_gstate * pgs, gs_memory_t * mem)
 {
     *pcdev = dev;		/* Do nothing -> return the same device */
     return 0;
@@ -868,7 +874,7 @@ gx_default_composite_clist_write_update(const gs_composite_t *pcte, gx_device *
 
 /* Default handler for adjusting a compositor's CTM. */
 int
-gx_default_composite_adjust_ctm(gs_composite_t *pcte, int x0, int y0, gs_imager_state *pis)
+gx_default_composite_adjust_ctm(gs_composite_t *pcte, int x0, int y0, gs_gstate *pgs)
 {
     return 0;
 }
@@ -897,7 +903,7 @@ gx_default_composite_is_friendly(const gs_composite_t *this, byte cmd0, byte cmd
  */
 int
 gx_default_composite_clist_read_update(gs_composite_t *pxcte, gx_device * cdev,
-                gx_device * tdev, gs_imager_state * pis, gs_memory_t * mem)
+                gx_device * tdev, gs_gstate * pgs, gs_memory_t * mem)
 {
     return 0;			/* Do nothing */
 }
@@ -943,15 +949,7 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
         case gxdso_is_std_cmyk_1bit:
             return (pdev->procs.map_cmyk_color == cmyk_1bit_map_cmyk_color);
         case gxdso_interpolate_antidropout:
-            if ((pdev->color_info.num_components == 1 &&
-                 pdev->color_info.max_gray < 15) ||
-                (pdev->color_info.num_components > 1 &&
-                 pdev->color_info.max_color < 15)) {
-                /* If we are are limited color device (i.e. we are halftoning)
-                 * then use the antidropout interpolators. */
-                return 1;
-            }
-            return 0;
+            return pdev->color_info.use_antidropout_downscaler;
         case gxdso_interpolate_threshold:
             if ((pdev->color_info.num_components == 1 &&
                  pdev->color_info.max_gray < 15) ||
@@ -968,16 +966,16 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
                 return gx_default_get_param(pdev, request->Param, request->list);
             }
     }
-    return gs_error_undefined;
+    return_error(gs_error_undefined);
 }
 
 int
 gx_default_fill_rectangle_hl_color(gx_device *pdev,
     const gs_fixed_rect *rect,
-    const gs_imager_state *pis, const gx_drawing_color *pdcolor,
+    const gs_gstate *pgs, const gx_drawing_color *pdcolor,
     const gx_clip_path *pcpath)
 {
-    return gs_error_rangecheck;
+    return_error(gs_error_rangecheck);
 }
 
 int
@@ -993,7 +991,7 @@ gx_default_include_color_space(gx_device *pdev, gs_color_space *cspace,
  * src/gsequivc.c.
  */
 int
-gx_default_update_spot_equivalent_colors(gx_device *pdev, const gs_state * pgs)
+gx_default_update_spot_equivalent_colors(gx_device *pdev, const gs_gstate * pgs)
 {
     return 0;
 }
@@ -1043,19 +1041,19 @@ gx_default_process_page(gx_device *dev, gx_process_page_options_t *options)
 /* ---------------- Default per-instance procedures ---------------- */
 
 int
-gx_default_install(gx_device * dev, gs_state * pgs)
+gx_default_install(gx_device * dev, gs_gstate * pgs)
 {
     return 0;
 }
 
 int
-gx_default_begin_page(gx_device * dev, gs_state * pgs)
+gx_default_begin_page(gx_device * dev, gs_gstate * pgs)
 {
     return 0;
 }
 
 int
-gx_default_end_page(gx_device * dev, int reason, gs_state * pgs)
+gx_default_end_page(gx_device * dev, int reason, gs_gstate * pgs)
 {
     return (reason != 2 ? 1 : 0);
 }
@@ -1285,7 +1283,10 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
     dev_to_subclass->procs.copy_planes = new_prototype->procs.copy_planes;
     dev_to_subclass->finalize = new_prototype->finalize;
     dev_to_subclass->dname = new_prototype->dname;
-    dev_to_subclass->icc_struct = NULL;
+    if (dev_to_subclass->icc_struct)
+        rc_increment(dev_to_subclass->icc_struct);
+    if (dev_to_subclass->PageList)
+        rc_increment(dev_to_subclass->PageList);
 
     /* In case the new device we're creating has already been initialised, copy
      * its additional data.
@@ -1294,6 +1295,11 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
     ptr1 = ((char *)new_prototype) + sizeof(gx_device);
     memcpy(ptr, ptr1, new_prototype->params_size - sizeof(gx_device));
 
+    /* We have to patch up the "type" parameters that the memory manage/garbage
+     * collector will use, as well.
+     */
+    (((obj_header_t *)dev_to_subclass) - 1)->o_type = new_prototype->stype;
+
     /* If the original device's stype structure was dynamically allocated, we need
      * to 'fixup' the contents, it's procs need to point to the new device's procs
      * for instance.
@@ -1361,6 +1367,10 @@ int gx_device_unsubclass(gx_device *dev)
      * their child pointer can then be NULL.
      */
     if (child) {
+        if (child->icc_struct)
+            rc_decrement(child->icc_struct, "gx_unsubclass_device, icc_struct");
+        if (child->PageList)
+            rc_decrement(child->PageList, "gx_unsubclass_device, PageList");
         /* we cannot afford to free the child device if its stype is not dynamic because
          * we can't 'null' the finalise routine, and we cannot permit the device to be finalised
          * because we have copied it up one level, not discarded it.
@@ -1372,6 +1382,7 @@ int gx_device_unsubclass(gx_device *dev)
             /* We definitley do *not* want to finalise the device, we just copied it up a level */
             b_std->finalize = 0;
             gs_free_object(dev->memory, child, "gx_unsubclass_device(device)");
+            child = 0;
         }
     }
     if(child)
@@ -1433,7 +1444,7 @@ int gx_update_from_subclass(gx_device *dev)
 }
 
 int gx_subclass_create_compositor(gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte,
-    gs_imager_state *pis, gs_memory_t *memory, gx_device *cdev)
+    gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev)
 {
     pdf14_clist_device *p14dev;
     generic_subclass_data *psubclass_data;
@@ -1462,13 +1473,13 @@ int gx_subclass_create_compositor(gx_device *dev, gx_device **pcdev, const gs_co
                     p14dev->target->child->procs.get_color_mapping_procs = p14dev->saved_target_get_color_mapping_procs;
                     p14dev->target->child->procs.get_color_comp_index = p14dev->saved_target_get_color_comp_index;
 
-                    pis->get_cmap_procs = p14dev->save_get_cmap_procs;
-                    gx_set_cmap_procs(pis, p14dev->target);
+                    pgs->get_cmap_procs = p14dev->save_get_cmap_procs;
+                    gx_set_cmap_procs(pgs, p14dev->target);
 
                     subclass_device = p14dev->target;
                     p14dev->target = p14dev->target->child;
 
-                    code = dev->procs.create_compositor(dev, pcdev, pcte, pis, memory, cdev);
+                    code = dev->procs.create_compositor(dev, pcdev, pcte, pgs, memory, cdev);
 
                     p14dev->target = subclass_device;
 
@@ -1476,11 +1487,11 @@ int gx_subclass_create_compositor(gx_device *dev, gx_device **pcdev, const gs_co
                 }
                 break;
             default:
-                code = dev->procs.create_compositor(dev, pcdev, pcte, pis, memory, cdev);
+                code = dev->procs.create_compositor(dev, pcdev, pcte, pgs, memory, cdev);
                 break;
         }
     } else {
-        code = dev->procs.create_compositor(dev, pcdev, pcte, pis, memory, cdev);
+        code = dev->procs.create_compositor(dev, pcdev, pcte, pgs, memory, cdev);
     }
     dev->procs.create_compositor = gx_subclass_create_compositor;
     return code;
