@@ -219,7 +219,7 @@ gs_font_dir_alloc2(gs_memory_t * struct_mem, gs_memory_t * bits_mem)
 {
     gs_font_dir *pdir = 0;
 
-#if !arch_small_memory
+#if !ARCH_SMALL_MEMORY
 #  ifdef DEBUG
     if (!gs_debug_c('.'))
 #  endif
@@ -264,7 +264,7 @@ gs_font_dir_alloc2_limits(gs_memory_t * struct_mem, gs_memory_t * bits_mem,
     pdir->smax = smax;
     pdir->align_to_pixels = false;
     pdir->glyph_to_unicode_table = NULL;
-    pdir->grid_fit_tt = 2;
+    pdir->grid_fit_tt = 1;
     pdir->memory = struct_mem;
     pdir->tti = 0;
     pdir->ttm = 0;
@@ -442,7 +442,7 @@ gs_font_find_similar(const gs_font_dir * pdir, const gs_font **ppfont,
 
 /* scalefont */
 int
-gs_scalefont(gs_font_dir * pdir, const gs_font * pfont, floatp scale,
+gs_scalefont(gs_font_dir * pdir, const gs_font * pfont, double scale,
              gs_font ** ppfont)
 {
     gs_matrix mat;
@@ -599,7 +599,7 @@ gs_makefont(gs_font_dir * pdir, const gs_font * pfont,
 /* Set the current font.  This is provided only for the benefit of cshow, */
 /* which must reset the current font without disturbing the root font. */
 void
-gs_set_currentfont(gs_state * pgs, gs_font * pfont)
+gs_set_currentfont(gs_gstate * pgs, gs_font * pfont)
 {
     pgs->font = pfont;
     pgs->char_tm_valid = false;
@@ -607,7 +607,7 @@ gs_set_currentfont(gs_state * pgs, gs_font * pfont)
 
 /* setfont */
 int
-gs_setfont(gs_state * pgs, gs_font * pfont)
+gs_setfont(gs_gstate * pgs, gs_font * pfont)
 {
     pgs->font = pgs->root_font = pfont;
     pgs->char_tm_valid = false;
@@ -616,14 +616,14 @@ gs_setfont(gs_state * pgs, gs_font * pfont)
 
 /* currentfont */
 gs_font *
-gs_currentfont(const gs_state * pgs)
+gs_currentfont(const gs_gstate * pgs)
 {
     return pgs->font;
 }
 
 /* rootfont */
 gs_font *
-gs_rootfont(const gs_state * pgs)
+gs_rootfont(const gs_gstate * pgs)
 {
     return pgs->root_font;
 }
@@ -643,7 +643,7 @@ gs_cachestatus(register const gs_font_dir * pdir, register uint pstat[7])
 
 /* setcacheparams */
 int
-gs_setcachesize(gs_state * pgs, gs_font_dir * pdir, uint size)
+gs_setcachesize(gs_gstate * pgs, gs_font_dir * pdir, uint size)
 {
     gs_memory_t *stable_mem = pdir->memory->stable_memory;
     if (size < 100000)             /* limits derived from CPSI emulation (CET 27-07) */
@@ -844,7 +844,7 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
         /*
          * Scan the glyph space to compute the fixed width if any.
          */
-        gs_glyph notdef = gs_no_glyph;
+        gs_glyph notdef = GS_NO_GLYPH;
         gs_glyph glyph;
         int fixed_width = 0;
         int index;
@@ -859,6 +859,7 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
              ) {
             gs_glyph_info_t glyph_info;
 
+            memset(&glyph_info, 0x00, sizeof(gs_glyph_info_t));
             code = font->procs.glyph_info(font, glyph, pmat,
                                           (GLYPH_INFO_WIDTH0 << wmode),
                                           &glyph_info);
@@ -866,7 +867,7 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
                 ecode = code;
                 continue;
             }
-            if (notdef == gs_no_glyph && gs_font_glyph_is_notdef(bfont, glyph)) {
+            if (notdef == GS_NO_GLYPH && gs_font_glyph_is_notdef(bfont, glyph)) {
                 notdef = glyph;
                 info->MissingWidth = (int)glyph_info.width[wmode].x;
                 info->members |= FONT_INFO_MISSING_WIDTH;
@@ -964,10 +965,10 @@ gs_font_glyph_is_notdef(gs_font_base *bfont, gs_glyph glyph)
 {
     gs_const_string gnstr;
 
-    if (glyph == gs_no_glyph)
+    if (glyph == GS_NO_GLYPH)
         return false;
-    if (glyph >= gs_min_cid_glyph)
-        return (glyph == gs_min_cid_glyph);
+    if (glyph >= GS_MIN_CID_GLYPH)
+        return (glyph == GS_MIN_CID_GLYPH);
     return (bfont->procs.glyph_name((gs_font *)bfont, glyph, &gnstr) >= 0 &&
             gnstr.size == 7 && !memcmp(gnstr.data, ".notdef", 7));
 }
@@ -976,12 +977,12 @@ gs_font_glyph_is_notdef(gs_font_base *bfont, gs_glyph glyph)
 gs_glyph
 gs_no_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t glyph_space)
 {
-    return gs_no_glyph;
+    return GS_NO_GLYPH;
 }
 
 /* Dummy glyph decoding procedure */
-gs_char
-gs_no_decode_glyph(gs_font *pfont, gs_glyph glyph, int ch)
+int
+gs_no_decode_glyph(gs_font *pfont, gs_glyph glyph, int ch, ushort *unicode_return, unsigned int length)
 {
     return GS_NO_CHAR;
 }
