@@ -41,13 +41,13 @@
 #include "gsicc_manage.h"
 
 /* Forward references */
-static gs_state *gstate_alloc(gs_memory_t *, client_name_t,
-                               const gs_state *);
-static gs_state *gstate_clone(gs_state *, gs_memory_t *, client_name_t,
-                               gs_state_copy_reason_t);
-static void gstate_free_contents(gs_state *);
-static int gstate_copy(gs_state *, const gs_state *,
-                        gs_state_copy_reason_t, client_name_t);
+static gs_gstate *gstate_alloc(gs_memory_t *, client_name_t,
+                               const gs_gstate *);
+static gs_gstate *gstate_clone(gs_gstate *, gs_memory_t *, client_name_t,
+                               gs_gstate_copy_reason_t);
+static void gstate_free_contents(gs_gstate *);
+static int gstate_copy(gs_gstate *, const gs_gstate *,
+                        gs_gstate_copy_reason_t, client_name_t);
 static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t cname);
 
 /*
@@ -137,7 +137,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  * individually for each state, except for line_params.dash.pattern.
  * Note that effective_clip_shared is not on the list.
  */
-typedef struct gs_state_parts_s {
+typedef struct gs_gstate_parts_s {
     gx_path *path;
     gx_clip_path *clip_path;
     gx_clip_path *effective_clip_path;
@@ -145,7 +145,7 @@ typedef struct gs_state_parts_s {
         gs_client_color *ccolor;
         gx_device_color *dev_color;
     } color[2];
-} gs_state_parts;
+} gs_gstate_parts;
 
 #define GSTATE_ASSIGN_PARTS(pto, pfrom)\
   ((pto)->path = (pfrom)->path, (pto)->clip_path = (pfrom)->clip_path,\
@@ -155,38 +155,13 @@ typedef struct gs_state_parts_s {
    (pto)->color[1].ccolor = (pfrom)->color[1].ccolor,\
    (pto)->color[1].dev_color = (pfrom)->color[1].dev_color)
 
-/* GC descriptors */
-extern_st(st_imager_state);
-public_st_gs_state();
-/* GC procedures for gs_state */
-static ENUM_PTRS_WITH(gs_state_enum_ptrs, gs_state *gsvptr)
-ENUM_PREFIX(st_imager_state, gs_state_num_ptrs + 2);
-#define e1(i,elt) ENUM_PTR(i,gs_state,elt);
-gs_state_do_ptrs(e1)
-case gs_state_num_ptrs: /* handle device specially */
-ENUM_RETURN(gx_device_enum_ptr(gsvptr->device));
-case gs_state_num_ptrs + 1:     /* handle device filter stack specially */
-ENUM_RETURN(gsvptr->dfilter_stack);
-#undef e1
-ENUM_PTRS_END
-static RELOC_PTRS_WITH(gs_state_reloc_ptrs, gs_state *gsvptr)
-{
-    RELOC_PREFIX(st_imager_state);
-    {
-#define r1(i,elt) RELOC_PTR(gs_state,elt);
-        gs_state_do_ptrs(r1)
-#undef r1
-        gsvptr->device = gx_device_reloc_ptr(gsvptr->device, gcst);
-        RELOC_PTR(gs_state, dfilter_stack);
-    }
-}
-RELOC_PTRS_END
+extern_st(st_gs_gstate); /* for gstate_alloc() */
 
 /* Copy client data, using the copy_for procedure if available, */
 /* the copy procedure otherwise. */
 static int
-gstate_copy_client_data(gs_state * pgs, void *dto, void *dfrom,
-                        gs_state_copy_reason_t reason)
+gstate_copy_client_data(gs_gstate * pgs, void *dto, void *dfrom,
+                        gs_gstate_copy_reason_t reason)
 {
     return (pgs->client_procs.copy_for != 0 ?
             (*pgs->client_procs.copy_for) (dto, dfrom, reason) :
@@ -195,11 +170,6 @@ gstate_copy_client_data(gs_state * pgs, void *dto, void *dfrom,
 
 /* ------ Operations on the entire graphics state ------ */
 
-/* Define the initial value of the graphics state. */
-static const gs_imager_state gstate_initial = {
-    gs_imager_state_initial(1.0, true)          /* is_gstate == true */
-};
-
 /*
  * Allocate a path for the graphics state.  We use stable memory because
  * some PostScript files have Type 3 fonts whose BuildChar procedure
@@ -214,16 +184,17 @@ gstate_path_memory(gs_memory_t *mem)
 }
 
 /* Allocate and initialize a graphics state. */
-gs_state *
-gs_state_alloc(gs_memory_t * mem)
+gs_gstate *
+gs_gstate_alloc(gs_memory_t * mem)
 {
-    gs_state *pgs = gstate_alloc(mem, "gs_state_alloc", NULL);
+    gs_gstate *pgs = gstate_alloc(mem, "gs_gstate_alloc", NULL);
+    gs_memory_t *path_mem = gstate_path_memory(mem);
     int code;
 
     if (pgs == 0)
         return 0;
-    *(gs_imager_state *)pgs = gstate_initial;   /* this sets is_gstate == true */
-    /* Need to set up at least enough to make gs_state_free happy */
+    GS_STATE_INIT_VALUES(pgs, 1.0);
+    /* Need to set up at least enough to make gs_gstate_free happy */
     pgs->saved = 0;
     pgs->path = NULL;
     pgs->clip_path = NULL;
@@ -238,25 +209,25 @@ gs_state_alloc(gs_memory_t * mem)
 
     /*
      * Just enough of the state is initialized at this point
-     * that it's OK to call gs_state_free if an allocation fails.
+     * that it's OK to call gs_gstate_free if an allocation fails.
      */
 
-    code = gs_imager_state_initialize((gs_imager_state *) pgs, mem);
+    code = gs_gstate_initialize(pgs, mem);
     if (code < 0)
         goto fail;
 
     /* Finish initializing the color rendering state. */
 
     rc_alloc_struct_1(pgs->halftone, gs_halftone, &st_halftone, mem,
-                      goto fail, "gs_state_alloc(halftone)");
+                      goto fail, "gs_gstate_alloc(halftone)");
     pgs->halftone->type = ht_type_none;
 
     /* Initialize other things not covered by initgraphics */
 
-    pgs->path = gx_path_alloc(gstate_path_memory(mem), "gs_state_alloc(path)");
-    pgs->clip_path = gx_cpath_alloc(mem, "gs_state_alloc(clip_path)");
+    pgs->path = gx_path_alloc(path_mem, "gs_gstate_alloc(path)");
+    pgs->clip_path = gx_cpath_alloc(path_mem, "gs_gstate_alloc(clip_path)");
     pgs->clip_stack = 0;
-    pgs->view_clip = gx_cpath_alloc(mem, "gs_state_alloc(view_clip)");
+    pgs->view_clip = gx_cpath_alloc(path_mem, "gs_gstate_alloc(view_clip)");
     if (pgs->view_clip == NULL)
         goto fail;
     pgs->view_clip->rule = 0;   /* no clipping */
@@ -301,15 +272,15 @@ gs_state_alloc(gs_memory_t * mem)
         return pgs;
     /* Something went very wrong. */
 fail:
-    gs_state_free(pgs);
+    gs_gstate_free(pgs);
     return 0;
 }
 
 /* Set the client data in a graphics state. */
 /* This should only be done to a newly created state. */
 void
-gs_state_set_client(gs_state * pgs, void *pdata,
-                    const gs_state_client_procs * pprocs, bool client_has_pattern_streams)
+gs_gstate_set_client(gs_gstate * pgs, void *pdata,
+                    const gs_gstate_client_procs * pprocs, bool client_has_pattern_streams)
 {
     pgs->client_data = pdata;
     pgs->client_procs = *pprocs;
@@ -317,22 +288,22 @@ gs_state_set_client(gs_state * pgs, void *pdata,
 }
 
 /* Get the client data from a graphics state. */
-#undef gs_state_client_data     /* gzstate.h makes this a macro */
+#undef gs_gstate_client_data     /* gzstate.h makes this a macro */
 void *
-gs_state_client_data(const gs_state * pgs)
+gs_gstate_client_data(const gs_gstate * pgs)
 {
     return pgs->client_data;
 }
 
 /* Free the chain of gstates.*/
 int
-gs_state_free_chain(gs_state * pgs)
+gs_gstate_free_chain(gs_gstate * pgs)
 {
-   gs_state *saved = pgs, *tmp;
+   gs_gstate *saved = pgs, *tmp;
 
    while(saved != 0) {
        tmp = saved->saved;
-       gs_state_free(saved);
+       gs_gstate_free(saved);
        saved = tmp;
    }
    return 0;
@@ -340,18 +311,18 @@ gs_state_free_chain(gs_state * pgs)
 
 /* Free a graphics state. */
 int
-gs_state_free(gs_state * pgs)
+gs_gstate_free(gs_gstate * pgs)
 {
     gstate_free_contents(pgs);
-    gs_free_object(pgs->memory, pgs, "gs_state_free");
+    gs_free_object(pgs->memory, pgs, "gs_gstate_free");
     return 0;
 }
 
 /* Save the graphics state. */
 int
-gs_gsave(gs_state * pgs)
+gs_gsave(gs_gstate * pgs)
 {
-    gs_state *pnew = gstate_clone(pgs, pgs->memory, "gs_gsave",
+    gs_gstate *pnew = gstate_clone(pgs, pgs->memory, "gs_gsave",
                                   copy_for_gsave);
 
     if (pnew == 0)
@@ -381,7 +352,7 @@ gs_gsave(gs_state * pgs)
  * In addition to an ordinary gsave, we create a new view clip path.
  */
 int
-gs_gsave_for_save(gs_state * pgs, gs_state ** psaved)
+gs_gsave_for_save(gs_gstate * pgs, gs_gstate ** psaved)
 {
     int code;
     gx_clip_path *old_cpath = pgs->view_clip;
@@ -414,9 +385,10 @@ fail:
 
 /* Restore the graphics state. Can fully empty graphics stack */
 int     /* return 0 if ok, 1 if stack was empty */
-gs_grestore_only(gs_state * pgs)
+gs_grestore_only(gs_gstate * pgs)
 {
-    gs_state *saved = pgs->saved;
+    gs_gstate *saved = pgs->saved;
+    gs_gstate tmp_gstate;
     void *pdata = pgs->client_data;
     void *sdata;
     bool prior_overprint = pgs->overprint;
@@ -434,9 +406,11 @@ gs_grestore_only(gs_state * pgs)
     if (pdata != 0 && sdata != 0)
         gstate_copy_client_data(pgs, pdata, sdata, copy_for_grestore);
     gstate_free_contents(pgs);
+    tmp_gstate = *pgs;              /* temp after contents freed (with pointers zeroed) */
     *pgs = *saved;
     if (pgs->show_gstate == saved)
         pgs->show_gstate = pgs;
+    *saved = tmp_gstate;            /* restore "freed" state (pointers zeroed after contents freed) */
     gs_free_object(pgs->memory, saved, "gs_grestore");
 
     /* update the overprint compositor, if necessary */
@@ -449,7 +423,7 @@ gs_grestore_only(gs_state * pgs)
 
 /* Restore the graphics state per PostScript semantics */
 int
-gs_grestore(gs_state * pgs)
+gs_grestore(gs_gstate * pgs)
 {
     int code;
     if (!pgs->saved)
@@ -467,7 +441,7 @@ gs_grestore(gs_state * pgs)
 /* Restore the graphics state for a 'restore', splicing the old stack */
 /* back on.  Note that we actually do a grestoreall + 2 grestores. */
 int
-gs_grestoreall_for_restore(gs_state * pgs, gs_state * saved)
+gs_grestoreall_for_restore(gs_gstate * pgs, gs_gstate * saved)
 {
     int code;
 
@@ -492,7 +466,7 @@ gs_grestoreall_for_restore(gs_state * pgs, gs_state * saved)
 
 /* Restore to the bottommost graphics state (at this save level). */
 int
-gs_grestoreall(gs_state * pgs)
+gs_grestoreall(gs_gstate * pgs)
 {
     if (!pgs->saved)            /* shouldn't happen */
         return gs_gsave(pgs);
@@ -506,15 +480,10 @@ gs_grestoreall(gs_state * pgs)
 }
 
 /* Allocate and return a new graphics state. */
-gs_state *
-gs_gstate(gs_state * pgs)
+gs_gstate *
+gs_gstate_copy(gs_gstate * pgs, gs_memory_t * mem)
 {
-    return gs_state_copy(pgs, pgs->memory);
-}
-gs_state *
-gs_state_copy(gs_state * pgs, gs_memory_t * mem)
-{
-    gs_state *pnew;
+    gs_gstate *pnew;
     /* Prevent 'capturing' the view clip path. */
     gx_clip_path *view_clip = pgs->view_clip;
 
@@ -522,7 +491,7 @@ gs_state_copy(gs_state * pgs, gs_memory_t * mem)
     pnew = gstate_clone(pgs, mem, "gs_gstate", copy_for_gstate);
     if (pnew == 0)
         return 0;
-    clip_stack_rc_adjust(pnew->clip_stack, 1, "gs_state_copy");
+    clip_stack_rc_adjust(pnew->clip_stack, 1, "gs_gstate_copy");
     rc_increment(pnew->dfilter_stack);
     pgs->view_clip = view_clip;
     pnew->saved = 0;
@@ -539,14 +508,14 @@ gs_state_copy(gs_state * pgs, gs_memory_t * mem)
 
 /* Copy one previously allocated graphics state to another. */
 int
-gs_copygstate(gs_state * pto, const gs_state * pfrom)
+gs_copygstate(gs_gstate * pto, const gs_gstate * pfrom)
 {
     return gstate_copy(pto, pfrom, copy_for_copygstate, "gs_copygstate");
 }
 
 /* Copy the current graphics state to a previously allocated one. */
 int
-gs_currentgstate(gs_state * pto, const gs_state * pgs)
+gs_currentgstate(gs_gstate * pto, const gs_gstate * pgs)
 {
     int code =
         gstate_copy(pto, pgs, copy_for_currentgstate, "gs_currentgstate");
@@ -558,14 +527,14 @@ gs_currentgstate(gs_state * pto, const gs_state * pgs)
 
 /* Restore the current graphics state from a previously allocated one. */
 int
-gs_setgstate(gs_state * pgs, const gs_state * pfrom)
+gs_setgstate(gs_gstate * pgs, const gs_gstate * pfrom)
 {
     /*
      * The implementation is the same as currentgstate,
      * except we must preserve the saved pointer, the level,
      * the view clip, and possibly the show_gstate.
      */
-    gs_state *saved_show = pgs->show_gstate;
+    gs_gstate *saved_show = pgs->show_gstate;
     int level = pgs->level;
     gx_clip_path *view_clip = pgs->view_clip;
     int code;
@@ -591,25 +560,25 @@ gs_setgstate(gs_state * pgs, const gs_state * pfrom)
 /* This is provided only for the interpreter */
 /* and for color space implementation. */
 gs_memory_t *
-gs_state_memory(const gs_state * pgs)
+gs_gstate_memory(const gs_gstate * pgs)
 {
     return pgs->memory;
 }
 
 /* Get the saved pointer of the graphics state. */
 /* This is provided only for Level 2 grestore. */
-gs_state *
-gs_state_saved(const gs_state * pgs)
+gs_gstate *
+gs_gstate_saved(const gs_gstate * pgs)
 {
     return pgs->saved;
 }
 
 /* Swap the saved pointer of the graphics state. */
 /* This is provided only for save/restore. */
-gs_state *
-gs_state_swap_saved(gs_state * pgs, gs_state * new_saved)
+gs_gstate *
+gs_gstate_swap_saved(gs_gstate * pgs, gs_gstate * new_saved)
 {
-    gs_state *saved = pgs->saved;
+    gs_gstate *saved = pgs->saved;
 
     pgs->saved = new_saved;
     return saved;
@@ -618,7 +587,7 @@ gs_state_swap_saved(gs_state * pgs, gs_state * new_saved)
 /* Swap the memory pointer of the graphics state. */
 /* This is provided only for the interpreter. */
 gs_memory_t *
-gs_state_swap_memory(gs_state * pgs, gs_memory_t * mem)
+gs_gstate_swap_memory(gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_memory_t *memory = pgs->memory;
 
@@ -635,26 +604,29 @@ gs_state_swap_memory(gs_state * pgs, gs_memory_t * mem)
  * compositor device.
  */
 int
-gs_state_update_overprint(gs_state * pgs, const gs_overprint_params_t * pparams)
+gs_gstate_update_overprint(gs_gstate * pgs, const gs_overprint_params_t * pparams)
 {
     gs_composite_t *    pct = 0;
-    gs_imager_state *   pis = (gs_imager_state *)pgs;
     int                 code;
     gx_device *         dev = pgs->device;
     gx_device *         ovptdev;
 
-    if ( (code = gs_create_overprint(&pct, pparams, pgs->memory)) >= 0 &&
-         (code = dev_proc(dev, create_compositor)( dev,
+    code = gs_create_overprint(&pct, pparams, pgs->memory);
+    if (code >= 0) {
+        code = dev_proc(dev, create_compositor)( dev,
                                                    &ovptdev,
                                                    pct,
-                                                   pis,
+                                                   pgs,
                                                    pgs->memory,
-                                                   NULL)) >= 0   ) {
-        if (ovptdev != dev)
-            gx_set_device_only(pgs, ovptdev);
+                                                   NULL);
+        if (code >= 0 || code == gs_error_handled){
+            if (ovptdev != dev)
+                gx_set_device_only(pgs, ovptdev);
+            code = 0;
+        }
     }
     if (pct != 0)
-        gs_free_object(pgs->memory, pct, "gs_state_update_overprint");
+        gs_free_object(pgs->memory, pct, "gs_gstate_update_overprint");
 
     /* the following hack handles devices that don't support compositors */
     if (code == gs_error_unknownerror && !pparams->retain_any_comps)
@@ -679,7 +651,7 @@ gs_state_update_overprint(gs_state * pgs, const gs_overprint_params_t * pparams)
  * components "drawn".
  */
 int
-gs_do_set_overprint(gs_state * pgs)
+gs_do_set_overprint(gs_gstate * pgs)
 {
     const gs_color_space *  pcs = gs_currentcolorspace_inline(pgs);
     const gs_client_color * pcc = gs_currentcolor_inline(pgs);
@@ -694,21 +666,10 @@ gs_do_set_overprint(gs_state * pgs)
 
 /* setoverprint */
 void
-gs_setoverprint(gs_state * pgs, bool ovp)
+gs_setoverprint(gs_gstate * pgs, bool ovp)
 {
     bool    prior_ovp = pgs->overprint;
-    cmm_dev_profile_t *profile_struct;
-    gx_device *dev = pgs->device;
 
-    /* Check if overprint is disabled */
-    if (dev != NULL) {
-        if (dev->procs.get_profile == NULL) {
-            profile_struct = dev->icc_struct;
-        } else {
-            dev_proc(dev, get_profile)(dev,  &profile_struct);
-        }
-        if (profile_struct != NULL && profile_struct->sim_overprint == false) return;
-    }
     pgs->overprint = ovp;
     if (prior_ovp != ovp)
         (void)gs_do_set_overprint(pgs);
@@ -716,14 +677,14 @@ gs_setoverprint(gs_state * pgs, bool ovp)
 
 /* currentoverprint */
 bool
-gs_currentoverprint(const gs_state * pgs)
+gs_currentoverprint(const gs_gstate * pgs)
 {
     return pgs->overprint;
 }
 
 /* setoverprintmode */
 int
-gs_setoverprintmode(gs_state * pgs, int mode)
+gs_setoverprintmode(gs_gstate * pgs, int mode)
 {
     int     prior_mode = pgs->effective_overprint_mode;
     int     code = 0;
@@ -738,7 +699,7 @@ gs_setoverprintmode(gs_state * pgs, int mode)
 
 /* currentoverprintmode */
 int
-gs_currentoverprintmode(const gs_state * pgs)
+gs_currentoverprintmode(const gs_gstate * pgs)
 {
     return pgs->overprint_mode;
 }
@@ -760,6 +721,23 @@ gs_currentcpsimode(const gs_memory_t * mem)
     return libctx->CPSI_mode;
 }
 
+void
+gs_setscanconverter(gs_memory_t *mem, int converter)
+{
+    gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
+
+    libctx->scanconverter = converter;
+}
+
+/* currentcpsimode */
+int
+gs_getscanconverter(const gs_memory_t * mem)
+{
+    gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
+
+    return libctx->scanconverter;
+}
+
 /* setrenderingintent
  *
  *  Use ICC numbers from Table 18 (section 6.1.11) rather than the PDF order
@@ -770,7 +748,7 @@ gs_currentcpsimode(const gs_memory_t * mem)
  *    AbsoluteColorimetric  3
  */
 int
-gs_setrenderingintent(gs_state *pgs, int ri) {
+gs_setrenderingintent(gs_gstate *pgs, int ri) {
     if (ri < 0 || ri > 3)
         return_error(gs_error_rangecheck);
     pgs->renderingintent = ri;
@@ -779,20 +757,20 @@ gs_setrenderingintent(gs_state *pgs, int ri) {
 
 /* currentrenderingintent */
 int
-gs_currentrenderingintent(const gs_state * pgs)
+gs_currentrenderingintent(const gs_gstate * pgs)
 {
     return pgs->renderingintent;
 }
 
 int
-gs_setblackptcomp(gs_state *pgs, bool bkpt) {
+gs_setblackptcomp(gs_gstate *pgs, bool bkpt) {
     pgs->blackptcomp = bkpt;
     return 0;
 }
 
 /* currentrenderingintent */
 bool
-gs_currentblackptcomp(const gs_state * pgs)
+gs_currentblackptcomp(const gs_gstate * pgs)
 {
     return pgs->blackptcomp;
 }
@@ -806,9 +784,12 @@ gs_currentblackptcomp(const gs_state * pgs)
  *     initializaion themselves.
  */
 int
-gs_initgraphics(gs_state * pgs)
+gs_initgraphics(gs_gstate * pgs)
 {
     int code;
+    const gs_gstate gstate_initial = {
+            gs_gstate_initial(1.0)
+        };
 
     gs_initmatrix(pgs);
     if ((code = gs_newpath(pgs)) < 0 ||
@@ -832,7 +813,7 @@ gs_initgraphics(gs_state * pgs)
 
 /* setfilladjust */
 int
-gs_setfilladjust(gs_state * pgs, double adjust_x, double adjust_y)
+gs_setfilladjust(gs_gstate * pgs, double adjust_x, double adjust_y)
 {
 #define CLAMP_TO_HALF(v)\
     ((v) <= 0 ? fixed_0 : (v) >= 0.5 ? fixed_half : float2fixed(v));
@@ -845,7 +826,7 @@ gs_setfilladjust(gs_state * pgs, double adjust_x, double adjust_y)
 
 /* currentfilladjust */
 int
-gs_currentfilladjust(const gs_state * pgs, gs_point * adjust)
+gs_currentfilladjust(const gs_gstate * pgs, gs_point * adjust)
 {
     adjust->x = fixed2float(pgs->fill_adjust.x);
     adjust->y = fixed2float(pgs->fill_adjust.y);
@@ -854,42 +835,42 @@ gs_currentfilladjust(const gs_state * pgs, gs_point * adjust)
 
 /* setlimitclamp */
 void
-gs_setlimitclamp(gs_state * pgs, bool clamp)
+gs_setlimitclamp(gs_gstate * pgs, bool clamp)
 {
     pgs->clamp_coordinates = clamp;
 }
 
 /* currentlimitclamp */
 bool
-gs_currentlimitclamp(const gs_state * pgs)
+gs_currentlimitclamp(const gs_gstate * pgs)
 {
     return pgs->clamp_coordinates;
 }
 
 /* settextrenderingmode */
 void
-gs_settextrenderingmode(gs_state * pgs, uint trm)
+gs_settextrenderingmode(gs_gstate * pgs, uint trm)
 {
     pgs->text_rendering_mode = trm;
 }
 
 /* currenttextrenderingmode */
 uint
-gs_currenttextrenderingmode(const gs_state * pgs)
+gs_currenttextrenderingmode(const gs_gstate * pgs)
 {
     return pgs->text_rendering_mode;
 }
 
 /* sethpglpathmode */
 void
-gs_sethpglpathmode(gs_state * pgs, bool path)
+gs_sethpglpathmode(gs_gstate * pgs, bool path)
 {
     pgs->hpgl_path_mode = path;
 }
 
 /* currenthpglpathmode */
 bool
-gs_currenthpglpathmode(const gs_state * pgs)
+gs_currenthpglpathmode(const gs_gstate * pgs)
 {
     return pgs->hpgl_path_mode;
 }
@@ -898,22 +879,31 @@ gs_currenthpglpathmode(const gs_state * pgs)
 
 /* Free the privately allocated parts of a gstate. */
 static void
-gstate_free_parts(const gs_state * parts, gs_memory_t * mem, client_name_t cname)
+gstate_free_parts(gs_gstate * parts, gs_memory_t * mem, client_name_t cname)
 {
     gs_free_object(mem, parts->color[1].dev_color, cname);
     gs_free_object(mem, parts->color[1].ccolor, cname);
     gs_free_object(mem, parts->color[0].dev_color, cname);
     gs_free_object(mem, parts->color[0].ccolor, cname);
-    if (!parts->effective_clip_shared)
+    parts->color[1].dev_color = 0;
+    parts->color[1].ccolor = 0;
+    parts->color[0].dev_color = 0;
+    parts->color[0].ccolor = 0;
+    if (!parts->effective_clip_shared && parts->effective_clip_path) {
         gx_cpath_free(parts->effective_clip_path, cname);
+        parts->effective_clip_path = 0;
+    }
     gx_cpath_free(parts->clip_path, cname);
-    if (parts->path)
+    parts->clip_path = 0;
+    if (parts->path) {
         gx_path_free(parts->path, cname);
+        parts->path = 0;
+    }
 }
 
 /* Allocate the privately allocated parts of a gstate. */
 static int
-gstate_alloc_parts(gs_state * parts, const gs_state * shared,
+gstate_alloc_parts(gs_gstate * parts, const gs_gstate * shared,
                    gs_memory_t * mem, client_name_t cname)
 {
     gs_memory_t *path_mem = gstate_path_memory(mem);
@@ -964,14 +954,15 @@ gstate_alloc_parts(gs_state * parts, const gs_state * shared,
  * clip_path and view_clip) effective_clip_path share the segments of
  * pfrom's corresponding path(s).
  */
-static gs_state *
-gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_state * pfrom)
+static gs_gstate *
+gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_gstate * pfrom)
 {
-    gs_state *pgs =
-        gs_alloc_struct(mem, gs_state, &st_gs_state, cname);
+    gs_gstate *pgs =
+        gs_alloc_struct(mem, gs_gstate, &st_gs_gstate, cname);
 
     if (pgs == 0)
         return 0;
+    memset(pgs, 0x00, sizeof(gs_gstate));
     if (gstate_alloc_parts(pgs, pfrom, mem, cname) < 0) {
         gs_free_object(mem, pgs, cname);
         return 0;
@@ -982,7 +973,7 @@ gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_state * pfrom)
 
 /* Copy the dash pattern from one gstate to another. */
 static int
-gstate_copy_dash(gs_state * pto, const gs_state * pfrom)
+gstate_copy_dash(gs_gstate * pto, const gs_gstate * pfrom)
 {
     return gs_setdash(pto, pfrom->line_params.dash.pattern,
                       pfrom->line_params.dash.pattern_size,
@@ -993,12 +984,12 @@ gstate_copy_dash(gs_state * pto, const gs_state * pfrom)
 /* Return 0 if the allocation fails. */
 /* If reason is for_gsave, the clone refers to the old contents, */
 /* and we switch the old state to refer to the new contents. */
-static gs_state *
-gstate_clone(gs_state * pfrom, gs_memory_t * mem, client_name_t cname,
-             gs_state_copy_reason_t reason)
+static gs_gstate *
+gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
+             gs_gstate_copy_reason_t reason)
 {
-    gs_state *pgs = gstate_alloc(mem, cname, pfrom);
-    gs_state_parts parts;
+    gs_gstate *pgs = gstate_alloc(mem, cname, pfrom);
+    gs_gstate_parts parts;
 
     if (pgs == 0)
         return 0;
@@ -1021,7 +1012,7 @@ gstate_clone(gs_state * pfrom, gs_memory_t * mem, client_name_t cname,
             )
             goto fail;
     }
-    gs_imager_state_copied((gs_imager_state *)pgs);
+    gs_gstate_copied(pgs);
     /* Don't do anything to clip_stack. */
 
     rc_increment(pgs->device);
@@ -1066,34 +1057,56 @@ clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t cname)
     }
 }
 
+/*
+ * Finalization for graphics states. This is where we handle RC for those
+ * elements.
+ */
+void
+gs_gstate_finalize(const gs_memory_t *cmem,void *vptr)
+{
+    gs_gstate *pgs = (gs_gstate *)vptr;
+    (void)cmem;	/* unused */
+
+    if (cmem == NULL)
+        return;			/* place for breakpoint */
+    gstate_free_contents(pgs);
+}
+
 /* Release the composite parts of a graphics state, */
 /* but not the state itself. */
 static void
-gstate_free_contents(gs_state * pgs)
+gstate_free_contents(gs_gstate * pgs)
 {
     gs_memory_t *mem = pgs->memory;
     const char *const cname = "gstate_free_contents";
 
     rc_decrement(pgs->device, cname);
+    pgs->device = 0;
     clip_stack_rc_adjust(pgs->clip_stack, -1, cname);
+    pgs->clip_stack = 0;
     rc_decrement(pgs->dfilter_stack, cname);
+    pgs->dfilter_stack = 0;
     gs_swapcolors_quick(pgs);
     cs_adjust_counts_icc(pgs, -1);
     gs_swapcolors_quick(pgs);
     cs_adjust_counts_icc(pgs, -1);
+    pgs->color[0].color_space = 0;
+    pgs->color[1].color_space = 0;
     if (pgs->client_data != 0)
         (*pgs->client_procs.free) (pgs->client_data, mem);
+    pgs->client_data = 0;
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
-    gstate_free_parts(pgs, mem, cname);
-    gs_imager_state_release((gs_imager_state *)pgs);
+    pgs->line_params.dash.pattern = 0;
+    gstate_free_parts(pgs, mem, cname);     /* this also clears pointers to freed elements */
+    gs_gstate_release(pgs);
 }
 
 /* Copy one gstate to another. */
 static int
-gstate_copy(gs_state * pto, const gs_state * pfrom,
-            gs_state_copy_reason_t reason, client_name_t cname)
+gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
+            gs_gstate_copy_reason_t reason, client_name_t cname)
 {
-    gs_state_parts parts;
+    gs_gstate_parts parts;
 
     GSTATE_ASSIGN_PARTS(&parts, pto);
     /* Copy the dash pattern if necessary. */
@@ -1145,11 +1158,10 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
         struct gx_pattern_cache_s *pcache = pto->pattern_cache;
         void *pdata = pto->client_data;
         gs_memory_t *mem = pto->memory;
-        gs_state *saved = pto->saved;
+        gs_gstate *saved = pto->saved;
         float *pattern = pto->line_params.dash.pattern;
 
-        gs_imager_state_pre_assign((gs_imager_state *)pto,
-                                   (const gs_imager_state *)pfrom);
+        gs_gstate_pre_assign(pto, (const gs_gstate *)pfrom);
         *pto = *pfrom;
         pto->client_data = pdata;
         pto->memory = mem;
@@ -1159,7 +1171,7 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
             pto->pattern_cache = pcache;
         if (pfrom->client_data != 0) {
             /* We need to break 'const' here. */
-            gstate_copy_client_data((gs_state *) pfrom, pdata,
+            gstate_copy_client_data((gs_gstate *) pfrom, pdata,
                                     pfrom->client_data, reason);
         }
     }
@@ -1174,12 +1186,12 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
 }
 
 /* Accessories. */
-gs_id gx_get_clip_path_id(gs_state *pgs)
+gs_id gx_get_clip_path_id(gs_gstate *pgs)
 {
     return pgs->clip_path->id;
 }
 
-void gs_swapcolors_quick(gs_state *pgs)
+void gs_swapcolors_quick(gs_gstate *pgs)
 {
     struct gx_cie_joint_caches_s *tmp_cie;
     gs_devicen_color_map          tmp_ccm;
@@ -1200,7 +1212,7 @@ void gs_swapcolors_quick(gs_state *pgs)
     pgs->color[0].color_space = pgs->color[1].color_space;
     pgs->color[1].color_space = tmp_cs;
 
-    /* Swap the bits of the imager state that depend on the current color */
+    /* Swap the bits of the gs_gstate that depend on the current color */
     tmp_cie                   = pgs->cie_joint_caches;
     pgs->cie_joint_caches     = pgs->cie_joint_caches_alt;
     pgs->cie_joint_caches_alt = tmp_cie;
@@ -1223,7 +1235,7 @@ void gs_swapcolors_quick(gs_state *pgs)
 
 }
 
-int gs_swapcolors(gs_state *pgs)
+int gs_swapcolors(gs_gstate *pgs)
 {
     int prior_overprint = pgs->overprint;
 
