@@ -24,7 +24,7 @@
 #include "gsstate.h"
 #include "gxdevice.h"
 #include "gxfixed.h"
-#include "gxistate.h"
+#include "gxgstate.h"
 #include "gzpath.h"
 #include "gxpaint.h"
 #include "gzcpath.h"
@@ -132,7 +132,7 @@ static const gx_device_cpath_accum gs_cpath_accum_device =
 
 /* Start accumulating a clipping path. */
 void
-gx_cpath_accum_begin(gx_device_cpath_accum * padev, gs_memory_t * mem)
+gx_cpath_accum_begin(gx_device_cpath_accum * padev, gs_memory_t * mem, bool transpose)
 {
     gx_device_init_on_stack((gx_device *) padev,
                             (const gx_device *) & gs_cpath_accum_device, mem);
@@ -140,21 +140,30 @@ gx_cpath_accum_begin(gx_device_cpath_accum * padev, gs_memory_t * mem)
     set_dev_proc(padev, encode_color, gx_default_gray_encode);
     set_dev_proc(padev, decode_color, gx_default_decode_color);
     (*dev_proc(padev, open_device)) ((gx_device *) padev);
+    padev->list.transpose = transpose;
 }
 
 void
 gx_cpath_accum_set_cbox(gx_device_cpath_accum * padev,
                         const gs_fixed_rect * pbox)
 {
-    padev->clip_box.p.x = fixed2int_var(pbox->p.x);
-    padev->clip_box.p.y = fixed2int_var(pbox->p.y);
-    padev->clip_box.q.x = fixed2int_var_ceiling(pbox->q.x);
-    padev->clip_box.q.y = fixed2int_var_ceiling(pbox->q.y);
+    if (padev->list.transpose) {
+        padev->clip_box.p.x = fixed2int_var(pbox->p.y);
+        padev->clip_box.p.y = fixed2int_var(pbox->p.x);
+        padev->clip_box.q.x = fixed2int_var_ceiling(pbox->q.y);
+        padev->clip_box.q.y = fixed2int_var_ceiling(pbox->q.x);
+    } else {
+        padev->clip_box.p.x = fixed2int_var(pbox->p.x);
+        padev->clip_box.p.y = fixed2int_var(pbox->p.y);
+        padev->clip_box.q.x = fixed2int_var_ceiling(pbox->q.x);
+        padev->clip_box.q.y = fixed2int_var_ceiling(pbox->q.y);
+    }
 }
 
 /* Finish accumulating a clipping path. */
+/* NB: After this the padev bbox will be restored to "normal" untransposed */
 int
-gx_cpath_accum_end(const gx_device_cpath_accum * padev, gx_clip_path * pcpath)
+gx_cpath_accum_end(gx_device_cpath_accum * padev, gx_clip_path * pcpath)
 {
     int code = (*dev_proc(padev, close_device)) ((gx_device *) padev);
     /* Make an entire clipping path so we can use cpath_assign. */
@@ -168,6 +177,16 @@ gx_cpath_accum_end(const gx_device_cpath_accum * padev, gx_clip_path * pcpath)
         apath.path.bbox.p.x = apath.path.bbox.p.y =
         apath.path.bbox.q.x = apath.path.bbox.q.y = 0;
     else {
+        if (padev->list.transpose) {
+            int tmp;
+
+            tmp = padev->bbox.p.x;
+            padev->bbox.p.x = padev->bbox.p.y;
+            padev->bbox.p.y = tmp;
+            tmp = padev->bbox.q.x;
+            padev->bbox.q.x = padev->bbox.q.y;
+            padev->bbox.q.y = tmp;
+        }
         apath.path.bbox.p.x = int2fixed(padev->bbox.p.x);
         apath.path.bbox.p.y = int2fixed(padev->bbox.p.y);
         apath.path.bbox.q.x = int2fixed(padev->bbox.q.x);
@@ -189,6 +208,7 @@ gx_cpath_accum_end(const gx_device_cpath_accum * padev, gx_clip_path * pcpath)
     gx_cpath_set_outer_box(&apath);
     apath.path_valid = false;
     apath.id = gs_next_ids(padev->list_memory, 1);	/* path changed => change id */
+    apath.cached = NULL;
     gx_cpath_assign_free(pcpath, &apath);
     return 0;
 }
@@ -203,33 +223,33 @@ gx_cpath_accum_discard(gx_device_cpath_accum * padev)
 /* Intersect two clipping paths using an accumulator. */
 int
 gx_cpath_intersect_path_slow(gx_clip_path * pcpath, gx_path * ppath,
-                             int rule, gs_imager_state *pis,
+                             int rule, gs_gstate *pgs,
                              const gx_fill_params * params0)
 {
-    gs_logical_operation_t save_lop = gs_current_logical_op_inline(pis);
+    gs_logical_operation_t save_lop = gs_current_logical_op_inline(pgs);
     gx_device_cpath_accum adev;
     gx_device_color devc;
     gx_fill_params params;
     int code;
 
-    gx_cpath_accum_begin(&adev, pcpath->path.memory);
+    gx_cpath_accum_begin(&adev, pcpath->path.memory, false);
     set_nonclient_dev_color(&devc, 0);	/* arbitrary, but not transparent */
-    gs_set_logical_op_inline(pis, lop_default);
+    gs_set_logical_op_inline(pgs, lop_default);
     if (params0 != 0)
         params = *params0;
     else {
         gs_point fadjust;
         params.rule = rule;
-        gs_currentfilladjust((gs_state *)pis, &fadjust);
+        gs_currentfilladjust(pgs, &fadjust);
         params.adjust.x = float2fixed(fadjust.x);
         params.adjust.y = float2fixed(fadjust.y);
-        params.flatness = gs_currentflat_inline(pis);
+        params.flatness = gs_currentflat_inline(pgs);
     }
-    code = gx_fill_path_only(ppath, (gx_device *)&adev, pis,
+    code = gx_fill_path_only(ppath, (gx_device *)&adev, pgs,
                              &params, &devc, pcpath);
     if (code < 0 || (code = gx_cpath_accum_end(&adev, pcpath)) < 0)
         gx_cpath_accum_discard(&adev);
-    gs_set_logical_op_inline(pis, save_lop);
+    gs_set_logical_op_inline(pgs, save_lop);
     return code;
 }
 
@@ -286,8 +306,13 @@ accum_close(gx_device * dev)
 {
     gx_device_cpath_accum * const adev = (gx_device_cpath_accum *)dev;
 
-    adev->list.xmin = adev->bbox.p.x;
-    adev->list.xmax = adev->bbox.q.x;
+    if (adev->list.transpose) {
+        adev->list.xmin = adev->bbox.p.y;
+        adev->list.xmax = adev->bbox.q.y;
+    } else {
+        adev->list.xmin = adev->bbox.p.x;
+        adev->list.xmax = adev->bbox.q.x;
+    }
 #ifdef DEBUG
     if (gs_debug_c('q')) {
         gx_clip_rect *rp =
@@ -423,16 +448,24 @@ accum_alloc_rect(gx_device_cpath_accum * adev)
  * this is possible.
  */
 static int
-accum_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
+accum_fill_rectangle(gx_device * dev, int xi, int yi, int w, int h,
                      gx_color_index color)
 {
     gx_device_cpath_accum * const adev = (gx_device_cpath_accum *)dev;
-    int xe = x + w, ye = y + h;
+    int x, y, xe, ye;
     gx_clip_rect *nr;
     gx_clip_rect *ar;
     register gx_clip_rect *rptr;
     int ymin, ymax;
 
+    if (adev->list.transpose) {
+        x = yi, xe = yi + h;
+        y = xi, ye = xi + w;
+    } else {
+        x = xi, xe = x + w;
+        y = yi, ye = y + h;
+    }
+
     /* Clip the rectangle being added. */
     if (y < adev->clip_box.p.y)
         y = adev->clip_box.p.y;
