@@ -31,7 +31,6 @@
 #include "gdevp14.h"
 #include "gsmemory.h"
 #include "gsmemlok.h"
-#include "vdtrace.h"
 #include "gsicc_cache.h"
 /*
  * We really don't like the fact that gdevprn.h is included here, since
@@ -229,7 +228,7 @@ buffer_segment_index(const stream_band_read_state *ss, uint buffer_offset, uint
             return i;
         }
     }
-    gs_note_error(gs_error_unregistered); /* Must not happen. */
+    (void)gs_note_error(gs_error_unregistered); /* Must not happen. */
     return -1;
 }
 
@@ -407,7 +406,10 @@ clist_find_pseudoband(gx_device_clist_reader *crdev, int band, cmd_block *cb)
     start_pos = page_info->bfile_end_pos - sizeof(cmd_block);
     page_info->io_procs->fseek(bfile, start_pos, SEEK_SET, page_info->bfname);
     while( 1 ) {
-        page_info->io_procs->fread_chars(cb, sizeof(cmd_block), bfile);
+        int read = page_info->io_procs->fread_chars(cb, sizeof(cmd_block), bfile);
+
+        if (read < sizeof(cmd_block))
+	    return -1;
         if (cb->band_max == band && cb->band_min == band) {
             page_info->io_procs->fseek(bfile, save_pos, SEEK_SET, page_info->bfname);
             return(0);  /* Found it */
@@ -579,7 +581,7 @@ clist_get_bits_rectangle(gx_device *dev, const gs_int_rect * prect,
     gs_int_rect band_rect;
     int lines_rasterized;
     gx_device *bdev;
-    int num_planes =
+    uint num_planes =
         (options & GB_PACKING_CHUNKY ? 1 :
          options & GB_PACKING_PLANAR ? dev->color_info.num_components :
          options & GB_PACKING_BIT_PLANAR ? dev->color_info.depth :
@@ -810,8 +812,8 @@ clist_render_rectangle(gx_device_clist *cldev, const gs_int_rect *prect,
 
             /* Store the page information. */
             page_info.cfile = page_info.bfile = NULL;
-            strncpy(page_info.cfname, ppage->page->cfname, sizeof(page_info.cfname));
-            strncpy(page_info.bfname, ppage->page->bfname, sizeof(page_info.bfname));
+            strncpy(page_info.cfname, ppage->page->cfname, sizeof(page_info.cfname)-1);
+            strncpy(page_info.bfname, ppage->page->bfname, sizeof(page_info.bfname)-1);
             page_info.io_procs = ppage->page->io_procs;
             page_info.tile_cache_size = ppage->page->tile_cache_size;
             page_info.bfile_end_pos = ppage->page->bfile_end_pos;
@@ -912,16 +914,7 @@ clist_playback_file_bands(clist_playback_action action,
         s.foreign = 1;
         s.state = (stream_state *)&rs;
 
-        if (vd_allowed('s')) {
-            vd_get_dc('s');
-        } else if (vd_allowed('i')) {
-            vd_get_dc('i');
-        }
-        vd_set_shift(0, 0);
-        vd_set_scale(0.01);
-        vd_set_origin(0, 0);
         code = clist_playback_band(action, crdev, &s, target, x0, y0, mem);
-        vd_release_dc;
 #	ifdef DEBUG
         s_band_read_dnit_offset_map(crdev, (stream_state *)&rs);
 #	endif
