@@ -119,9 +119,9 @@ typedef struct gx_device_s gx_device;
 
 /* We need at least an abstract type for a graphics state, */
 /* which is passed to the page device procedures. */
-#ifndef gs_state_DEFINED
-#  define gs_state_DEFINED
-typedef struct gs_state_s gs_state;
+#ifndef gs_gstate_DEFINED
+#  define gs_gstate_DEFINED
+typedef struct gs_gstate_s gs_gstate;
 #endif
 
 /* We need abstract types for paths and fill/stroke parameters, */
@@ -142,9 +142,9 @@ typedef struct gx_fill_params_s gx_fill_params;
 #  define gx_stroke_params_DEFINED
 typedef struct gx_stroke_params_s gx_stroke_params;
 #endif
-#ifndef gs_imager_state_DEFINED
-#  define gs_imager_state_DEFINED
-typedef struct gs_imager_state_s gs_imager_state;
+#ifndef gs_gstate_DEFINED
+#  define gs_gstate_DEFINED
+typedef struct gs_gstate_s gs_gstate;
 #endif
 #ifndef patch_fill_state_t_DEFINED
 #  define patch_fill_state_t_DEFINED
@@ -255,16 +255,11 @@ typedef enum {
  * information is required, hence the use of an enumeration with an
  * "unknown" setting.
  *
- * GX_CINFO_OPMODE_RGB is an odd case where by the device is RGB based 
- * but we attempt to simulate CMY overprinting.  GC_CINFO_OPMODE_RGB_SET
- * is the value after we verify the color model and the colorant positions
  */
 typedef enum {
     GX_CINFO_OPMODE_UNKNOWN = -1,
     GX_CINFO_OPMODE_NOT = 0,
-    GX_CINFO_OPMODE = 1,
-    GX_CINFO_OPMODE_RGB,
-    GC_CINFO_OPMODE_RGB_SET
+    GX_CINFO_OPMODE = 1
 } gx_cm_opmode_t;
 
 /* component index value used to indicate no color component.  */
@@ -301,7 +296,7 @@ typedef struct gx_device_color_info_s {
      * color models supported by this device. This does not include
      * any alpha components.
      */
-    int max_components;
+    uchar max_components;
 
     /*
      * The number of color components. This does not include any
@@ -309,7 +304,7 @@ typedef struct gx_device_color_info_s {
      * the gx_color_index but is otherwise passed as a separate
      * component.
      */
-    int num_components;
+    uchar num_components;
 
     /*
      * Polarity of the components of the color space, either
@@ -321,7 +316,7 @@ typedef struct gx_device_color_info_s {
 
     /*
      * The number of bits of gx_color_index actually used.
-     * This must be <= arch_sizeof_color_index, which is usually 64.
+     * This must be <= ARCH_SIZEOF_COLOR_INDEX, which is usually 64.
      * Note that we now have planar devices which can support much more.
      * Changing this to a ushort to reflect this.
      */
@@ -451,7 +446,8 @@ typedef struct gx_device_color_info_s {
      */
     gx_cm_opmode_t opmode;
     gx_color_index process_comps;
-    int black_component;\
+    uint black_component;
+    bool use_antidropout_downscaler;
 } gx_device_color_info;
 
 /* NB encoding flag ignored */
@@ -620,15 +616,15 @@ typedef struct gx_device_procs_s gx_device_procs;
 typedef struct gx_page_device_procs_s {
 
 #define dev_page_proc_install(proc)\
-  int proc(gx_device *dev, gs_state *pgs)
+  int proc(gx_device *dev, gs_gstate *pgs)
     dev_page_proc_install((*install));
 
 #define dev_page_proc_begin_page(proc)\
-  int proc(gx_device *dev, gs_state *pgs)
+  int proc(gx_device *dev, gs_gstate *pgs)
     dev_page_proc_begin_page((*begin_page));
 
 #define dev_page_proc_end_page(proc)\
-  int proc(gx_device *dev, int reason, gs_state *pgs)
+  int proc(gx_device *dev, int reason, gs_gstate *pgs)
     dev_page_proc_end_page((*end_page));
 
 } gx_page_device_procs;
@@ -694,9 +690,10 @@ typedef struct gx_band_params_s {
     int BandWidth;		/* (optional) band width in pixels */
     int BandHeight;		/* (optional) */
     long BandBufferSpace;	/* (optional) */
+    long tile_cache_size;	/* (optional) */
 } gx_band_params_t;
 
-#define BAND_PARAMS_INITIAL_VALUES 0, 0, 0
+#define BAND_PARAMS_INITIAL_VALUES 0, 0, 0, 0
 
 typedef enum {
     BandingAuto = 0,
@@ -715,6 +712,12 @@ typedef struct gdev_space_params_s {
     gdev_banding_type banding_type;	/* used to force banding or bitmap */
 } gdev_space_params;
 
+typedef struct gdev_pagelist_s {
+        rc_header rc;
+        char *Pages;
+        int PagesSize;
+} gdev_pagelist;
+
 #define gx_device_common\
         int params_size;		/* OBSOLETE if stype != 0: */\
                                         /* size of this structure */\
@@ -734,6 +737,7 @@ typedef struct gdev_space_params_s {
         gx_device *parent;\
         gx_device *child;\
         void *subclass_data;    /* Must be immovable, non-GC memory, used to store subclass data */\
+        gdev_pagelist *PageList;\
         bool is_open;			/* true if device has been opened */\
         int max_fill_band;		/* limit on band size for fill, */\
                                         /* must be 0 or a power of 2 */\
@@ -750,12 +754,9 @@ typedef struct gdev_space_params_s {
         float ImagingBBox[4];		/* imageable region in points */\
         bool ImagingBBox_set;\
         float HWResolution[2];		/* resolution, dots per inch */\
-        float MarginsHWResolution[2];	/* resolution for Margins. The pagedevice */\
-                                        /* documentation implies these units might be */\
-                                        /* different than the physical device units. */\
         float Margins[2];		/* offset of physical page corner */\
                                         /* from device coordinate (0,0), */\
-                                        /* in units given by MarginsHWResolution */\
+                                        /* in units given by HWResolution */\
         float HWMargins[4];		/* margins around imageable area, */\
                                         /* in default user units ("points") */\
         int FirstPage;\
@@ -773,6 +774,7 @@ typedef struct gdev_space_params_s {
         bool LockSafetyParams;		/* If true, prevent unsafe changes */\
         long band_offset_x;		/* offsets of clist band base to (mem device) buffer */\
         long band_offset_y;		/* for rendering that is phase sensitive (old wtsimdi) */\
+        bool BLS_force_memory;\
         gx_stroked_gradient_recognizer_t sgr;\
         int MaxPatternBitmap;		/* Threshold for switching to pattern_clist mode */\
         bool page_uses_transparency;    /* PDF 1.4 transparency is used. */\
@@ -812,8 +814,8 @@ typedef struct gdev_space_params_s {
 #define no_margins margin_values(0, 0, 0, 0)
 #define no_margins_() no_margins
 /* Define macros that give the page offset ("Margins") in inches. */
-#define dev_x_offset(dev) ((dev)->Margins[0] / (dev)->MarginsHWResolution[0])
-#define dev_y_offset(dev) ((dev)->Margins[1] / (dev)->MarginsHWResolution[1])
+#define dev_x_offset(dev) ((dev)->Margins[0] / (dev)->HWResolution[0])
+#define dev_y_offset(dev) ((dev)->Margins[1] / (dev)->HWResolution[1])
 #define dev_y_offset_points(dev) (dev_y_offset(dev) * 72.0)
 /* Note that left/right/top/bottom are defined relative to */
 /* the physical paper, not the coordinate system. */
@@ -1032,7 +1034,7 @@ typedef struct gs_param_list_s gs_param_list;
 
 #define dev_t_proc_fill_path(proc, dev_t)\
   int proc(dev_t *dev,\
-    const gs_imager_state *pis, gx_path *ppath,\
+    const gs_gstate *pgs, gx_path *ppath,\
     const gx_fill_params *params,\
     const gx_drawing_color *pdcolor, const gx_clip_path *pcpath)
 #define dev_proc_fill_path(proc)\
@@ -1040,7 +1042,7 @@ typedef struct gs_param_list_s gs_param_list;
 
 #define dev_t_proc_stroke_path(proc, dev_t)\
   int proc(dev_t *dev,\
-    const gs_imager_state *pis, gx_path *ppath,\
+    const gs_gstate *pgs, gx_path *ppath,\
     const gx_stroke_params *params,\
     const gx_drawing_color *pdcolor, const gx_clip_path *pcpath)
 #define dev_proc_stroke_path(proc)\
@@ -1101,7 +1103,7 @@ typedef struct gs_param_list_s gs_param_list;
 
 #define dev_t_proc_begin_image(proc, dev_t)\
   int proc(dev_t *dev,\
-    const gs_imager_state *pis, const gs_image_t *pim,\
+    const gs_gstate *pgs, const gs_image_t *pim,\
     gs_image_format_t format, const gs_int_rect *prect,\
     const gx_drawing_color *pdcolor, const gx_clip_path *pcpath,\
     gs_memory_t *memory, gx_image_enum_common_t **pinfo)
@@ -1156,7 +1158,7 @@ typedef struct gs_param_list_s gs_param_list;
 
 #define dev_t_proc_begin_typed_image(proc, dev_t)\
   int proc(dev_t *dev,\
-    const gs_imager_state *pis, const gs_matrix *pmat,\
+    const gs_gstate *pgs, const gs_matrix *pmat,\
     const gs_image_common_t *pim, const gs_int_rect *prect,\
     const gx_drawing_color *pdcolor, const gx_clip_path *pcpath,\
     gs_memory_t *memory, gx_image_enum_common_t **pinfo)
@@ -1180,7 +1182,7 @@ typedef struct gs_param_list_s gs_param_list;
 #define dev_t_proc_create_compositor(proc, dev_t)\
   int proc(dev_t *dev,\
     gx_device **pcdev, const gs_composite_t *pcte,\
-    gs_imager_state *pis, gs_memory_t *memory, gx_device *cdev)
+    gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev)
 #define dev_proc_create_compositor(proc)\
   dev_t_proc_create_compositor(proc, gx_device)\
 
@@ -1212,7 +1214,7 @@ typedef struct gs_param_list_s gs_param_list;
 /*
   Push the current transparency state (*ppts) onto the associated stack,
   and set *ppts to a new transparency state of the given dimension.  The
-  transparency state may copy some or all of the imager state, such as the
+  transparency state may copy some or all of the gs_gstate, such as the
   current alpha and/or transparency mask values, and definitely copies the
   parameters.
 */
@@ -1220,7 +1222,7 @@ typedef struct gs_param_list_s gs_param_list;
   int proc(gx_device *dev,\
     const gs_transparency_group_params_t *ptgp,\
     const gs_rect *pbbox,\
-    gs_imager_state *pis,\
+    gs_gstate *pgs,\
     gs_memory_t *mem)
 #define dev_proc_begin_transparency_group(proc)\
   dev_t_proc_begin_transparency_group(proc, gx_device)
@@ -1234,7 +1236,7 @@ typedef struct gs_param_list_s gs_param_list;
 */
 #define dev_t_proc_end_transparency_group(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis)
+    gs_gstate *pgs)
 #define dev_proc_end_transparency_group(proc)\
   dev_t_proc_end_transparency_group(proc, gx_device)
 
@@ -1247,7 +1249,7 @@ typedef struct gs_param_list_s gs_param_list;
   int proc(gx_device *dev,\
     const gx_transparency_mask_params_t *ptmp,\
     const gs_rect *pbbox,\
-    gs_imager_state *pis,\
+    gs_gstate *pgs,\
     gs_memory_t *mem)
 #define dev_proc_begin_transparency_mask(proc)\
   dev_t_proc_begin_transparency_mask(proc, gx_device)
@@ -1255,12 +1257,12 @@ typedef struct gs_param_list_s gs_param_list;
 /*
   Store a pointer to the rendered transparency mask into *pptm, popping the
   stack like end_group.  Normally, the client will follow this by using
-  rc_assign to store the rendered mask into pis->{opacity,shape}.mask.  If
+  rc_assign to store the rendered mask into pgs->{opacity,shape}.mask.  If
   end_mask fails, the stack is *not* popped.
 */
 #define dev_t_proc_end_transparency_mask(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis)
+    gs_gstate *pgs)
 #define dev_proc_end_transparency_mask(proc)\
   dev_t_proc_end_transparency_mask(proc, gx_device)
 
@@ -1271,7 +1273,7 @@ typedef struct gs_param_list_s gs_param_list;
 */
 #define dev_t_proc_discard_transparency_layer(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis)
+    gs_gstate *pgs)
 #define dev_proc_discard_transparency_layer(proc)\
   dev_t_proc_discard_transparency_layer(proc, gx_device)
 
@@ -1334,14 +1336,14 @@ typedef enum {
   a proper code.
 
   Currently this function is used with gs_rectfill and gs_fillpage.  It is
-  also used for the handling of the devn color type for supporting 
-  large number of spot colorants to planar separation devices. 
-  
+  also used for the handling of the devn color type for supporting
+  large number of spot colorants to planar separation devices.
+
 */
 
 #define dev_t_proc_fill_rectangle_hl_color(proc, dev_t)\
   int proc(dev_t *dev, const gs_fixed_rect *rect, \
-        const gs_imager_state *pis, const gx_drawing_color *pdcolor, \
+        const gs_gstate *pgs, const gx_drawing_color *pdcolor, \
         const gx_clip_path *pcpath)
 #define dev_proc_fill_rectangle_hl_color(proc)\
   dev_t_proc_fill_rectangle_hl_color(proc, gx_device)
@@ -1430,7 +1432,7 @@ typedef struct gs_fill_attributes_s {
  * start of src/gsequivc.c.
  */
 #define dev_t_proc_update_spot_equivalent_colors(proc, dev_t)\
-  int proc(dev_t *dev, const gs_state * pgs)
+  int proc(dev_t *dev, const gs_gstate * pgs)
 #define dev_proc_update_spot_equivalent_colors(proc)\
   dev_t_proc_update_spot_equivalent_colors(proc, gx_device)
 
@@ -1453,25 +1455,25 @@ typedef struct gs_devn_params_s gs_devn_params;
  */
 
 #define dev_t_proc_fillpage(proc, dev_t)\
-  int proc(gx_device *dev, gs_imager_state * pis, gx_device_color *pdevc)
+  int proc(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
 #define dev_proc_fillpage(proc)\
   dev_t_proc_fillpage(proc, gx_device)
 
 #define dev_t_proc_push_transparency_state(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis)
+    gs_gstate *pgs)
 #define dev_proc_push_transparency_state(proc)\
   dev_t_proc_push_transparency_state(proc, gx_device)
 
 #define dev_t_proc_pop_transparency_state(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis)
+    gs_gstate *pgs)
 #define dev_proc_pop_transparency_state(proc)\
   dev_t_proc_pop_transparency_state(proc, gx_device)
 
 #define dev_t_proc_put_image(proc, dev_t)\
-  int proc(gx_device *dev, const byte *buffer, int num_chan, int x, int y,\
-            int width, int height, int row_stride, int plane_stride,\
+  int proc(gx_device *dev, const byte **buffers, int num_chan, int x, int y,\
+            int width, int height, int row_stride,\
             int alpha_plane_index, int tag_plane_index)
 #define dev_proc_put_image(proc)\
   dev_t_proc_put_image(proc, gx_device)
@@ -1636,12 +1638,12 @@ typedef struct gx_image_plane_s {
     uint raster;
 } gx_image_plane_t;
 
-#define gx_device_begin_image(dev, pis, pim, format, prect, pdcolor, pcpath, memory, pinfo)\
+#define gx_device_begin_image(dev, pgs, pim, format, prect, pdcolor, pcpath, memory, pinfo)\
   ((*dev_proc(dev, begin_image))\
-   (dev, pis, pim, format, prect, pdcolor, pcpath, memory, pinfo))
-#define gx_device_begin_typed_image(dev, pis, pmat, pim, prect, pdcolor, pcpath, memory, pinfo)\
+   (dev, pgs, pim, format, prect, pdcolor, pcpath, memory, pinfo))
+#define gx_device_begin_typed_image(dev, pgs, pmat, pim, prect, pdcolor, pcpath, memory, pinfo)\
   ((*dev_proc(dev, begin_typed_image))\
-   (dev, pis, pmat, pim, prect, pdcolor, pcpath, memory, pinfo))
+   (dev, pgs, pmat, pim, prect, pdcolor, pcpath, memory, pinfo))
 
 /*
  * The driver-like procedures gx_device_{image_data, image_plane_data,
@@ -1766,7 +1768,7 @@ gx_cm_color_map_procs *get_color_mapping_procs_subclass(const gx_device *dev)
 }
 
 static inline
-void map_rgb_subclass(const gx_cm_color_map_procs *procs, gx_device *dev, const gs_imager_state *pis, frac r, frac g, frac b, frac out[])
+void map_rgb_subclass(const gx_cm_color_map_procs *procs, gx_device *dev, const gs_gstate *pgs, frac r, frac g, frac b, frac out[])
 {
     if (dev == NULL)
         return;
@@ -1774,7 +1776,7 @@ void map_rgb_subclass(const gx_cm_color_map_procs *procs, gx_device *dev, const
     {
         dev = dev->parent;
     }
-    procs->map_rgb(dev, pis, r, g, b, out);
+    procs->map_rgb(dev, pgs, r, g, b, out);
 }
 
 static inline
@@ -1892,7 +1894,7 @@ void gx_device_set_media_size(gx_device * dev, double media_width, double media_
  * Temporarily install a null device, or a special device such as
  * a clipping or cache device.
  */
-void gx_set_device_only(gs_state *, gx_device *);
+void gx_set_device_only(gs_gstate *, gx_device *);
 
 /* Close a device. */
 int gs_closedevice(gx_device *);
@@ -1920,4 +1922,7 @@ const gx_device_type dtname = { &stype, initproc }
 void gx_device_dump(gx_device *dev, const char *text);
 #endif
 
+/* Compare color information structures */
+bool gx_color_info_equal(const gx_device_color_info *p1, const gx_device_color_info *p2);
+
 #endif /* gxdevcli_INCLUDED */
