@@ -64,7 +64,7 @@
 #endif
 
 /* Define parameters for machines with little dinky RAMs.... */
-#if arch_small_memory
+#if ARCH_SMALL_MEMORY
 #   define MAX_BITMAP 32000
 #   define BUFFER_SPACE 25000
 #   define MIN_MEMORY_LEFT 32000
@@ -98,10 +98,11 @@
  * Note also that the macro does not initialize procs, which is
  * the next element of the structure.
  */
-#define std_device_part1_(devtype, ptr_procs, dev_name, stype, open_init)\
+    #define std_device_part1_(devtype, ptr_procs, dev_name, stype, open_init)\
         sizeof(devtype), ptr_procs, dev_name,\
         0 /*memory*/, stype, 0 /*stype_is_dynamic*/, 0 /*finalize*/,\
-        { 0 } /*rc*/, 0 /*retained*/, 0 /* parent */, 0 /* child */, 0 /* subclass_data */, open_init() /*is_open, max_fill_band*/
+        { 0 } /*rc*/, 0 /*retained*/, 0 /* parent */, 0 /* child */, 0 /* subclass_data */, 0, /* PageList */\
+        open_init() /*is_open, max_fill_band*/
         /* color_info goes here */
 /*
  * The MetroWerks compiler has some bizarre bug that produces a spurious
@@ -114,14 +115,15 @@
         { (float)((((width) * 72.0 + 0.5) - 0.5) / (x_dpi))/*MediaSize[0]*/,\
           (float)((((height) * 72.0 + 0.5) - 0.5) / (y_dpi))/*MediaSize[1]*/},\
         { 0, 0, 0, 0 }/*ImagingBBox*/, 0/*ImagingBBox_set*/,\
-        { x_dpi, y_dpi }/*HWResolution*/, { x_dpi, y_dpi }/*MarginsHWResolution*/
+        { x_dpi, y_dpi }/*HWResolution*/
 
 /* offsets and margins go here */
 #define std_device_part3_()\
         0/*FirstPage*/, 0/*LastPage*/, 0/*PageHandlerPushed*/, 0/*DisablePageHandler*/, 0/* Object Filter*/, 0/*ObjectHandlerPushed*/,\
         0/*PageCount*/, 0/*ShowpageCount*/, 1/*NumCopies*/, 0/*NumCopies_set*/,\
         0/*IgnoreNumCopies*/, 0/*UseCIEColor*/, 0/*LockSafetyParams*/,\
-        0/*band_offset_x*/, 0/*band_offset_y*/, {false}/* sgr */,\
+        0/*band_offset_x*/, 0/*band_offset_y*/, false /*BLS_force_memory*/, \
+        {false}/* sgr */,\
         0/* MaxPatternBitmap */, 0/*page_uses_transparency*/,\
         { MAX_BITMAP, BUFFER_SPACE,\
              { BAND_PARAMS_INITIAL_VALUES },\
@@ -494,7 +496,7 @@ int gx_device_delete_output_file(const gx_device * dev, const char *fname);
 
 /*
  * Determine whether a given device needs to halftone.  Eventually this
- * should take an imager state as an additional argument.
+ * should take a gs_gstate as an additional argument.
  */
 #define gx_device_must_halftone(dev)\
   ((gx_device_has_color(dev) ? (dev)->color_info.max_color :\
@@ -644,7 +646,7 @@ int gs_is_pdf14trans_compositor(const gs_composite_t * pct);
 #define subclass_common\
     t_dev_proc_create_compositor *saved_compositor_method
 
-typedef int (t_dev_proc_create_compositor) (gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte, gs_imager_state *pis, gs_memory_t *memory, gx_device *cdev);
+typedef int (t_dev_proc_create_compositor) (gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte, gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev);
 
 typedef struct {
     t_dev_proc_create_compositor *saved_compositor_method;
@@ -656,7 +658,7 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
 int gx_device_unsubclass(gx_device *dev);
 int gx_update_from_subclass(gx_device *dev);
 int gx_subclass_create_compositor(gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte,
-    gs_imager_state *pis, gs_memory_t *memory, gx_device *cdev);
+    gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev);
 
 
 #endif /* gxdevice_INCLUDED */
