@@ -23,12 +23,6 @@
 
 ******************************************************************************/
 
-/* Configuration management identification */
-#ifndef lint
-static const char
-  cm_id[] = "@(#)$Id: gdevpcl3.c,v 1.32 2001/08/14 15:22:35 Martin Rel $";
-#endif
-
 /*****************************************************************************/
 
 #ifndef _XOPEN_SOURCE
@@ -127,7 +121,7 @@ static const ms_MediaCode
 /*****************************************************************************/
 
 /* Forward declaration */
-static void pcl3_flag_mismatch_reporter(FILE *err,
+static void pcl3_flag_mismatch_reporter(
   const struct s_eprn_Device *eprn, bool no_match);
 
 /* Macro for creating device structure instances */
@@ -428,7 +422,7 @@ static void init(pcl3_Device *dev)
 
 ******************************************************************************/
 
-static void pcl3_flag_mismatch_reporter(FILE *err,
+static void pcl3_flag_mismatch_reporter(
   const struct s_eprn_Device *eprn, bool no_match)
 {
   const char *epref = eprn->CUPS_messages? CUPS_ERRPREF: "";
@@ -908,7 +902,8 @@ static int pcl3_put_params(gx_device *device, gs_param_list *plist)
       else {
         eprintf1("%s" ERRPREF "Unknown subdevice name: `", epref);
         errwrite(dev->memory,
-                 string_value.data, sizeof(char)*string_value.size);
+                 (const char *)string_value.data,
+                 sizeof(char)*string_value.size);
         eprintf("'.\n");
         last_error = gs_error_rangecheck;
         param_signal_error(plist, pname, last_error);
@@ -1018,7 +1013,8 @@ static int pcl3_put_params(gx_device *device, gs_param_list *plist)
       else {
         eprintf1("%s" ERRPREF "Invalid duplex capability: `", epref);
         errwrite(dev->memory,
-                 string_value.data, sizeof(char)*string_value.size);
+                 (const char *)string_value.data,
+                 sizeof(char)*string_value.size);
         eprintf("'.\n");
         last_error = gs_error_rangecheck;
         param_signal_error(plist, pname, last_error);
@@ -1074,7 +1070,8 @@ static int pcl3_put_params(gx_device *device, gs_param_list *plist)
       if (rc != gs_error_VMerror) {
         eprintf1("%s" ERRPREF "Unknown medium: `", epref);
         errwrite(dev->memory,
-                 string_value.data, sizeof(char)*string_value.size);
+                 (const char *)string_value.data,
+                 sizeof(char)*string_value.size);
         eprintf("'.\n");
       }
       last_error = rc;
@@ -1144,7 +1141,8 @@ static int pcl3_put_params(gx_device *device, gs_param_list *plist)
       if (rc != gs_error_VMerror) {
         eprintf1("%s" ERRPREF "Unknown print quality: `", epref);
         errwrite(dev->memory,
-                 string_value.data, sizeof(char)*string_value.size);
+                 (const char *)string_value.data,
+                 sizeof(char)*string_value.size);
         eprintf("'.\n");
       }
       last_error = rc;
@@ -1232,8 +1230,9 @@ static int pcl3_put_params(gx_device *device, gs_param_list *plist)
 
   /* Process parameters defined by base classes (should occur after treating
      parameters defined for the derived class, see gsparam.h) */
-  if ((rc = eprn_put_params(device, plist)) < 0 ||
-    rc > 0 && last_error >= 0) last_error = rc;
+  rc = eprn_put_params(device, plist);
+  if (rc < 0 || (rc > 0 && last_error >= 0))
+    last_error = rc;
 
   /* Act if the colour model was changed */
   if (previous_colour_model != dev->eprn.colour_model) set_palette(dev);
