@@ -17,6 +17,7 @@
 /* PhotoShop (PSD) export device, supporting DeviceN color models. */
 
 #include "math_.h"
+#include "gxiodev.h"
 #include "gdevprn.h"
 #include "gsparam.h"
 #include "gscrd.h"
@@ -34,6 +35,7 @@
 #include "gdevppla.h"
 #include "gxdownscale.h"
 #include "gdevdevnprn.h"
+#include "gdevpsd.h"
 
 #ifndef cmm_gcmmhlink_DEFINED
     #define cmm_gcmmhlink_DEFINED
@@ -49,9 +51,6 @@
 #   define MAX_CHAN 15
 #endif
 
-/* Enable logic for a local ICC output profile. */
-#define ENABLE_ICC_PROFILE 0
-
 /* Define the device parameters. */
 #ifndef X_DPI
 #  define X_DPI 72
@@ -65,6 +64,8 @@ static dev_proc_open_device(psd_prn_open);
 static dev_proc_close_device(psd_prn_close);
 static dev_proc_get_params(psd_get_params);
 static dev_proc_put_params(psd_put_params);
+static dev_proc_get_params(psd_get_params_cmyk);
+static dev_proc_put_params(psd_put_params_cmyk);
 static dev_proc_print_page(psd_print_page);
 static dev_proc_map_color_rgb(psd_map_color_rgb);
 static dev_proc_get_color_mapping_procs(get_psdrgb_color_mapping_procs);
@@ -88,10 +89,11 @@ typedef struct psd_device_s {
 
     psd_color_model color_model;
 
-    long downscale_factor;
     int max_spots;
     bool lock_colorants;
 
+    gx_downscaler_params downscale;
+
     /* ICC color profile objects, for color conversion.
        These are all device link profiles.  At least that
        is how it appears looking at how this code
@@ -121,6 +123,7 @@ static
 ENUM_PTRS_WITH(psd_device_enum_ptrs, psd_device *pdev)
 {
     ENUM_PREFIX(st_gx_devn_prn_device, 0);
+    (void)pdev; /* Stop unused var warning */
     return 0;
 }
 ENUM_PTRS_END
@@ -128,6 +131,7 @@ ENUM_PTRS_END
 static RELOC_PTRS_WITH(psd_device_reloc_ptrs, psd_device *pdev)
 {
     RELOC_PREFIX(st_gx_devn_prn_device);
+    (void)pdev; /* Stop unused var warning */
 }
 RELOC_PTRS_END
 
@@ -147,7 +151,7 @@ gs_private_st_composite_final(st_psd_device, psd_device,
 /*
  * Macro definition for psd device procedures
  */
-#define device_procs(get_color_mapping_procs)\
+#define device_procs(get_color_mapping_procs, params)\
 {	psd_prn_open,\
         gx_default_get_initial_matrix,\
         NULL,				/* sync_output */\
@@ -162,8 +166,8 @@ gs_private_st_composite_final(st_psd_device, psd_device,
         NULL,				/* copy_color */\
         NULL,				/* draw_line */\
         NULL,				/* get_bits */\
-        psd_get_params,			/* get_params */\
-        psd_put_params,			/* put_params */\
+        psd_get_##params,		/* get_params */\
+        psd_put_##params,		/* put_params */\
         NULL,				/* map_cmyk_color - not used */\
         NULL,				/* get_xfont_procs */\
         NULL,				/* get_xfont_device */\
@@ -247,7 +251,7 @@ static fixed_colorant_name DeviceRGBComponents[] = {
  * PSD device with RGB process color model.
  */
 static const gx_device_procs spot_rgb_procs =
-    device_procs(get_psdrgb_color_mapping_procs);
+    device_procs(get_psdrgb_color_mapping_procs, params);
 
 const psd_device gs_psdrgb_device =
 {
@@ -265,16 +269,16 @@ const psd_device gs_psdrgb_device =
     { true },			/* equivalent CMYK colors for spot colors */
     /* PSD device specific parameters */
     psd_DEVICE_RGB,		/* Color model */
-    1,                          /* downscale_factor */
     GS_SOFT_MAX_SPOTS,           /* max_spots */
     false,                      /* colorants not locked */
+    GX_DOWNSCALER_PARAMS_DEFAULTS
 };
 
 /*
  * PSD device with CMYK process color model and spot color support.
  */
 static const gx_device_procs spot_cmyk_procs
-        = device_procs(get_psd_color_mapping_procs);
+        = device_procs(get_psd_color_mapping_procs, params_cmyk);
 
 const psd_device gs_psdcmyk_device =
 {
@@ -296,9 +300,9 @@ const psd_device gs_psdcmyk_device =
     { true },			/* equivalent CMYK colors for spot colors */
     /* PSD device specific parameters */
     psd_DEVICE_CMYK,		/* Color model */
-    1,                          /* downscale_factor */
-    GS_SOFT_MAX_SPOTS,           /* max_spots */
+    GS_SOFT_MAX_SPOTS,          /* max_spots */
     false,                      /* colorants not locked */
+    GX_DOWNSCALER_PARAMS_DEFAULTS
 };
 
 /* Open the psd devices */
@@ -354,12 +358,19 @@ psd_prn_open(gx_device * pdev)
         }
     }
     pdev_psd->warning_given = false;
+
+    /* For the planar device we need to set up the bit depth of each plane.
+       For other devices this is handled in check_device_separable where
+       we compute the bit shift for the components etc. */
+    for (k = 0; k < GS_CLIENT_COLOR_MAX_COMPONENTS; k++) {
+        pdev->color_info.comp_bits[k] = 8;
+    }
+
     /* With planar the depth can be more than 64.  Update the color
        info to reflect the proper depth and number of planes.  Also note
        that the number of spot colors can change from page to page.  
        Update things so that we only output separations for the
        inks on that page. */
-
     if (pdev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE) {
         if ((pdev_psd->devn_params.page_spot_colors >= 0 || force_pdf) && !force_ps) {
             if (force_pdf) {
@@ -423,7 +434,7 @@ gray_cs_to_psdgray_cm(gx_device * dev, frac gray, frac out[])
 }
 
 static void
-rgb_cs_to_psdgray_cm(gx_device * dev, const gs_imager_state *pis,
+rgb_cs_to_psdgray_cm(gx_device * dev, const gs_gstate *pgs,
                                    frac r, frac g, frac b, frac out[])
 {
     out[0] = color_rgb_to_gray(r, g, b, NULL);
@@ -450,7 +461,7 @@ gray_cs_to_psdrgb_cm(gx_device * dev, frac gray, frac out[])
 }
 
 static void
-rgb_cs_to_psdrgb_cm(gx_device * dev, const gs_imager_state *pis,
+rgb_cs_to_psdrgb_cm(gx_device * dev, const gs_gstate *pgs,
                                   frac r, frac g, frac b, frac out[])
 {
     int i = ((psd_device *)dev)->devn_params.separations.num_separations;
@@ -484,12 +495,12 @@ gray_cs_to_psdcmyk_cm(gx_device * dev, frac gray, frac out[])
 }
 
 static void
-rgb_cs_to_psdcmyk_cm(gx_device * dev, const gs_imager_state *pis,
+rgb_cs_to_psdcmyk_cm(gx_device * dev, const gs_gstate *pgs,
                            frac r, frac g, frac b, frac out[])
 {
     int * map = ((psd_device *) dev)->devn_params.separation_order_map;
 
-    rgb_cs_to_devn_cm(dev, map, pis, r, g, b, out);
+    rgb_cs_to_devn_cm(dev, map, pgs, r, g, b, out);
 }
 
 static void
@@ -571,7 +582,7 @@ gray_cs_to_spotn_cm(gx_device * dev, frac gray, frac out[])
 }
 
 static void
-rgb_cs_to_spotn_cm(gx_device * dev, const gs_imager_state *pis,
+rgb_cs_to_spotn_cm(gx_device * dev, const gs_gstate *pgs,
                                    frac r, frac g, frac b, frac out[])
 {
     psd_device *xdev = (psd_device *)dev;
@@ -600,7 +611,7 @@ rgb_cs_to_spotn_cm(gx_device * dev, const gs_imager_state *pis,
     } else {
         frac cmyk[4];
 
-        color_rgb_to_cmyk(r, g, b, pis, cmyk, dev->memory);
+        color_rgb_to_cmyk(r, g, b, pgs, cmyk, dev->memory);
         cmyk_cs_to_spotn_cm(dev, cmyk[0], cmyk[1], cmyk[2], cmyk[3],
                             out);
     }
@@ -666,106 +677,19 @@ psd_map_color_rgb(gx_device *dev, gx_color_index color, gx_color_value rgb[3])
     return 0;
 }
 
-#if ENABLE_ICC_PROFILE
-static int
-psd_open_profile(const char *profile_out_fn, cmm_profile_t *icc_profile, gcmmhlink_t icc_link, gs_memory_t *memory)
-{
-
-    gsicc_rendering_param_t rendering_params;
-
-    icc_profile = gsicc_get_profile_handle_file(profile_out_fn,
-                    strlen(profile_out_fn), memory);
-
-    if (icc_profile == NULL)
-        return gs_throw(-1, "Could not create profile for psd device");
-
-    /* Set up the rendering parameters */
-
-    rendering_params.black_point_comp = gsBPNOTSPECIFIED;
-    rendering_params.graphics_type_tag = GS_UNKNOWN_TAG;  /* Already rendered */
-    rendering_params.rendering_intent = gsPERCEPTUAL;
-
-    /* Call with a NULL destination profile since we are using a device link profile here */
-    icc_link = gscms_get_link(icc_profile,
-                    NULL, &rendering_params);
-
-    if (icc_link == NULL)
-        return gs_throw(-1, "Could not create link handle for psd device");
-
-    return(0);
-
-}
-
-static int
-psd_open_profiles(psd_device *xdev)
-{
-    int code = 0;
-
-    if (xdev->output_icc_link == NULL && xdev->profile_out_fn[0]) {
-
-        code = psd_open_profile(xdev->profile_out_fn, xdev->output_profile,
-            xdev->output_icc_link, xdev->memory);
-
-    }
-
-    if (code >= 0 && xdev->rgb_icc_link == NULL && xdev->profile_rgb_fn[0]) {
-
-        code = psd_open_profile(xdev->profile_rgb_fn, xdev->rgb_profile,
-            xdev->rgb_icc_link, xdev->memory);
-
-    }
-
-    if (code >= 0 && xdev->cmyk_icc_link == NULL && xdev->profile_cmyk_fn[0]) {
-
-        code = psd_open_profile(xdev->profile_cmyk_fn, xdev->cmyk_profile,
-            xdev->cmyk_icc_link, xdev->memory);
-
-    }
-
-    return code;
-
-}
-#endif
-
 /* Get parameters.  We provide a default CRD. */
 static int
-psd_get_params(gx_device * pdev, gs_param_list * plist)
+psd_get_params_generic(gx_device * pdev, gs_param_list * plist, int cmyk)
 {
     psd_device *xdev = (psd_device *)pdev;
     int code;
-#if ENABLE_ICC_PROFILE
-    gs_param_string pos;
-    gs_param_string prgbs;
-    gs_param_string pcmyks;
-#endif
 
     code = gx_devn_prn_get_params(pdev, plist);
     if (code < 0)
         return code;
 
-#if ENABLE_ICC_PROFILE
-    pos.data = (const byte *)xdev->profile_out_fn,
-        pos.size = strlen(xdev->profile_out_fn),
-        pos.persistent = false;
-    code = param_write_string(plist, "ProfileOut", &pos);
-    if (code < 0)
-        return code;
-
-    prgbs.data = (const byte *)xdev->profile_rgb_fn,
-        prgbs.size = strlen(xdev->profile_rgb_fn),
-        prgbs.persistent = false;
-    code = param_write_string(plist, "ProfileRgb", &prgbs);
-    if (code < 0)
-        return code;
-
-    pcmyks.data = (const byte *)xdev->profile_cmyk_fn,
-        pcmyks.size = strlen(xdev->profile_cmyk_fn),
-        pcmyks.persistent = false;
-    code = param_write_string(plist, "ProfileCmyk", &prgbs);
-    if (code < 0)
-        return code;
-#endif
-    code = param_write_long(plist, "DownScaleFactor", &xdev->downscale_factor);
+    code = gx_downscaler_write_params(plist, &xdev->downscale,
+                                      cmyk ? GX_DOWNSCALER_PARAMS_TRAP : 0);
     if (code < 0)
         return code;
     code = param_write_int(plist, "MaxSpots", &xdev->max_spots);
@@ -775,22 +699,19 @@ psd_get_params(gx_device * pdev, gs_param_list * plist)
     return code;
 }
 
-#if ENABLE_ICC_PROFILE
 static int
-psd_param_read_fn(gs_param_list *plist, const char *name,
-                  gs_param_string *pstr, uint max_len)
+psd_get_params(gx_device * pdev, gs_param_list * plist)
 {
-    int code = param_read_string(plist, name, pstr);
-
-    if (code == 0) {
-        if (pstr->size >= max_len)
-            param_signal_error(plist, name, code = gs_error_rangecheck);
-    } else {
-        pstr->data = 0;
-    }
-    return code;
+    return psd_get_params_generic(pdev, plist, 0);
 }
-#endif
+
+static int
+psd_get_params_cmyk(gx_device * pdev, gs_param_list * plist)
+{
+    return psd_get_params_generic(pdev, plist, 1);
+}
+
+
 
 /* Compare a C string and a gs_param_string. */
 static bool
@@ -833,32 +754,18 @@ psd_set_color_model(psd_device *xdev, psd_color_model color_model)
 
 /* Set parameters.  We allow setting the number of bits per component. */
 static int
-psd_put_params(gx_device * pdev, gs_param_list * plist)
+psd_put_params_generic(gx_device * pdev, gs_param_list * plist, int cmyk)
 {
     psd_device * const pdevn = (psd_device *) pdev;
     int code = 0;
-#if ENABLE_ICC_PROFILE
-    gs_param_string po;
-    gs_param_string prgb;
-    gs_param_string pcmyk;
-#endif
     gs_param_string pcm;
     psd_color_model color_model = pdevn->color_model;
     gx_device_color_info save_info = pdevn->color_info;
 
-    switch (code = param_read_long(plist,
-                                   "DownScaleFactor",
-                                   &pdevn->downscale_factor)) {
-        case 0:
-            if (pdevn->downscale_factor <= 0)
-                pdevn->downscale_factor = 1;
-            break;
-        case 1:
-            break;
-        default:
-            param_signal_error(plist, "DownScaleFactor", code);
-            return code;
-    }
+    code = gx_downscaler_read_params(plist, &pdevn->downscale,
+                                     cmyk ? GX_DOWNSCALER_PARAMS_TRAP : 0);
+    if (code < 0)
+        return code;
 
     switch (code = param_read_bool(plist, "LockColorants", &(pdevn->lock_colorants))) {
         case 0:
@@ -878,7 +785,7 @@ psd_put_params(gx_device * pdev, gs_param_list * plist)
                 break;
             emprintf1(pdevn->memory, "MaxSpots must be between 0 and %d\n",
                       GS_CLIENT_COLOR_MAX_COMPONENTS-4);
-            code = gs_error_rangecheck;
+            code = gs_note_error(gs_error_rangecheck);
             /* fall through */
         default:
             param_signal_error(plist, "MaxSpots", code);
@@ -887,17 +794,6 @@ psd_put_params(gx_device * pdev, gs_param_list * plist)
             break;
     }
 
-#if ENABLE_ICC_PROFILE
-    code = psd_param_read_fn(plist, "ProfileOut", &po,
-                                 sizeof(pdevn->profile_out_fn));
-    if (code >= 0)
-        code = psd_param_read_fn(plist, "ProfileRgb", &prgb,
-                                 sizeof(pdevn->profile_rgb_fn));
-    if (code >= 0)
-        code = psd_param_read_fn(plist, "ProfileCmyk", &pcmyk,
-                                 sizeof(pdevn->profile_cmyk_fn));
-#endif
-
     if (code >= 0)
         code = param_read_name(plist, "ProcessColorModel", &pcm);
     if (code == 0) {
@@ -927,27 +823,21 @@ psd_put_params(gx_device * pdev, gs_param_list * plist)
         return code;
     }
 
-#if ENABLE_ICC_PROFILE
-    /* Open any ICC profiles that have been specified. */
-    if (po.data != 0) {
-        memcpy(pdevn->profile_out_fn, po.data, po.size);
-        pdevn->profile_out_fn[po.size] = 0;
-    }
-    if (prgb.data != 0) {
-        memcpy(pdevn->profile_rgb_fn, prgb.data, prgb.size);
-        pdevn->profile_rgb_fn[prgb.size] = 0;
-    }
-    if (pcmyk.data != 0) {
-        memcpy(pdevn->profile_cmyk_fn, pcmyk.data, pcmyk.size);
-        pdevn->profile_cmyk_fn[pcmyk.size] = 0;
-    }
-    if (memcmp(&pdevn->color_info, &save_info,
-                            size_of(gx_device_color_info)) != 0)
-        code = psd_open_profiles(pdevn);
-#endif
     return code;
 }
 
+static int
+psd_put_params(gx_device * pdev, gs_param_list * plist)
+{
+    return psd_put_params_generic(pdev, plist, 0);
+}
+
+static int
+psd_put_params_cmyk(gx_device * pdev, gs_param_list * plist)
+{
+    return psd_put_params_generic(pdev, plist, 1);
+}
+
 /*
  * This routine will check to see if the color component name  match those
  * that are available amoung the current device's color components.
@@ -990,7 +880,7 @@ psd_get_color_comp_index(gx_device * dev, const char * pname,
 /* ------ Private definitions ------ */
 
 /* All two-byte quantities are stored MSB-first! */
-#if arch_is_big_endian
+#if ARCH_IS_BIG_ENDIAN
 #  define assign_u16(a,v) a = (v)
 #  define assign_u32(a,v) a = (v)
 #else
@@ -1322,8 +1212,11 @@ psd_write_image_data(psd_write_ctx *xc, gx_device_printer *pdev)
     if (sep_line == NULL)
         return_error(gs_error_VMerror);
 
-    code = gx_downscaler_init_planar(&ds, (gx_device *)pdev, &params, num_comp,
-                                     psd_dev->downscale_factor, 0, 8, 8);
+    code = gx_downscaler_init_planar_trapped(&ds, (gx_device *)pdev, &params, num_comp,
+                                             psd_dev->downscale.downscale_factor, 0, 8, 8,
+                                             psd_dev->downscale.trap_w,
+                                             psd_dev->downscale.trap_h,
+                                             psd_dev->downscale.trap_order);
     if (code < 0)
         goto cleanup;
 
@@ -1374,17 +1267,51 @@ cleanup:
     return code;
 }
 
+bool
+psd_allow_multiple_pages (gx_device_printer *pdev)
+{
+    psd_device *psd_dev = (psd_device *)pdev;
+    int code;
+    const char *fmt;
+    bool retval = true;
+    gs_parsed_file_name_t parsed;
+
+    if (strcmp(gp_null_file_name, psd_dev->fname) == 0) {
+        /* If the output is discarded, it doesn't matter whether
+         * the file is format valid or not
+         */
+        retval = true;
+    }
+    else {
+        code = gx_parse_output_file_name(&parsed, &fmt, psd_dev->fname,
+                                     strlen(psd_dev->fname), psd_dev->memory);
+        if (code < 0 || (fmt == NULL && psd_dev->PageCount > 0)) {
+            retval = false;
+        }
+    }
+    return retval;
+}
+
 static int
 psd_print_page(gx_device_printer *pdev, FILE *file)
 {
     psd_write_ctx xc;
     gx_devn_prn_device *devn_dev = (gx_devn_prn_device *)pdev;
     psd_device *psd_dev = (psd_device *)pdev;
+    int code = 0;
 
-    psd_setup(&xc, devn_dev, file,
-              gx_downscaler_scale(pdev->width, psd_dev->downscale_factor),
-              gx_downscaler_scale(pdev->height, psd_dev->downscale_factor));
-    psd_write_header(&xc, devn_dev);
-    psd_write_image_data(&xc, pdev);
-    return 0;
+    if (!psd_allow_multiple_pages(pdev)) {
+       emprintf(pdev->memory, "Use of the %%d format is required to output more than one page to PSD\nSee doc/Devices.htm#PSD for details\n\n");
+       code = gs_note_error(gs_error_ioerror);
+    }
+    else {
+        code = psd_setup(&xc, devn_dev, file,
+                  gx_downscaler_scale(pdev->width, psd_dev->downscale.downscale_factor),
+                  gx_downscaler_scale(pdev->height, psd_dev->downscale.downscale_factor));
+        if (code >= 0)
+            code = psd_write_header(&xc, devn_dev);
+        if (code >= 0)
+            code = psd_write_image_data(&xc, pdev);
+    }
+    return code;
 }
