@@ -650,10 +650,10 @@ pdf_obj_mark_unused(gx_device_pdf *pdev, long id)
 
     if (gp_fseek_64 (tfile, ((int64_t)(id - pdev->FirstObjectNumber)) * sizeof(pos),
           SEEK_SET) != 0)
-          return gs_error_ioerror;
+      return_error(gs_error_ioerror);
     fwrite(&pos, sizeof(pos), 1, tfile);
     if (gp_fseek_64(tfile, tpos, SEEK_SET) != 0)
-        return gs_error_ioerror;
+      return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -672,10 +672,10 @@ pdf_open_obj(gx_device_pdf * pdev, long id, pdf_resource_type_t type)
 
         if (gp_fseek_64 (tfile, ((int64_t)(id - pdev->FirstObjectNumber)) * sizeof(pos),
               SEEK_SET) != 0)
-              return gs_error_ioerror;
+	  return_error(gs_error_ioerror);
         fwrite(&pos, sizeof(pos), 1, tfile);
         if (gp_fseek_64(tfile, tpos, SEEK_SET) != 0)
-              return gs_error_ioerror;
+	  return_error(gs_error_ioerror);
     }
     if (pdev->ForOPDFRead && pdev->ProduceDSC) {
         switch(type) {
@@ -702,7 +702,7 @@ pdf_open_obj(gx_device_pdf * pdev, long id, pdf_resource_type_t type)
             case resourceCIDFont:
             case resourceFont:
                 /* Ought to write the font name here */
-                pprintld1(s, "%%%%BeginResource: font (PDF Font obj_%ld)\n", id);
+                pprintld1(s, "%%%%BeginResource: procset (PDF Font obj_%ld)\n", id);
                 break;
             case resourceCharProc:
                 pprintld1(s, "%%%%BeginResource: file (PDF CharProc obj_%ld)\n", id);
@@ -1295,7 +1295,7 @@ pdf_find_resource_by_resource_id(gx_device_pdf * pdev, pdf_resource_type_t rtype
 
     for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
         for (pres = pchain[i]; pres != 0; pres = pres->next) {
-            if (pres->object->id == id)
+            if (pres->object && pres->object->id == id)
                 return pres;
         }
     }
@@ -1338,6 +1338,53 @@ pdf_find_same_resource(gx_device_pdf * pdev, pdf_resource_type_t rtype, pdf_reso
     return 0;
 }
 
+void
+pdf_drop_resource_from_chain(gx_device_pdf * pdev, pdf_resource_t *pres1, pdf_resource_type_t rtype)
+{
+    pdf_resource_t **pchain = pdev->resources[rtype].chains;
+    pdf_resource_t *pres;
+    pdf_resource_t **pprev = &pdev->last_resource;
+    int i;
+
+    /* since we're about to free the resource, we can just set
+       any of these references to null
+    */
+    for (i = 0; i < pdev->sbstack_size; i++) {
+        if (pres1 == pdev->sbstack[i].font3) {
+            pdev->sbstack[i].font3 = NULL;
+        }
+        else if (pres1 == pdev->sbstack[i].accumulating_substream_resource) {
+            pdev->sbstack[i].accumulating_substream_resource = NULL;
+        }
+        else if (pres1 == pdev->sbstack[i].pres_soft_mask_dict) {
+            pdev->sbstack[i].pres_soft_mask_dict = NULL;
+        }
+    }
+
+    for (; (pres = *pprev) != 0; pprev = &pres->prev)
+        if (pres == pres1) {
+            *pprev = pres->prev;
+            break;
+        }
+
+    for (i = (gs_id_hash(pres1->rid) % NUM_RESOURCE_CHAINS); i < NUM_RESOURCE_CHAINS; i++) {
+        pprev = pchain + i;
+        for (; (pres = *pprev) != 0; pprev = &pres->next)
+            if (pres == pres1) {
+                *pprev = pres->next;
+#if 0
+                if (pres->object) {
+                    COS_RELEASE(pres->object, "pdf_forget_resource");
+                    gs_free_object(pdev->pdf_memory, pres->object, "pdf_forget_resource");
+                    pres->object = 0;
+                }
+                gs_free_object(pdev->pdf_memory, pres, "pdf_forget_resource");
+#endif
+                return;
+            }
+    }
+}
+
 /* Drop resources by a condition. */
 void
 pdf_drop_resources(gx_device_pdf * pdev, pdf_resource_type_t rtype,
@@ -1696,7 +1743,7 @@ pdf_store_page_resources(gx_device_pdf *pdev, pdf_page_t *page, bool clear_usage
 }
 
 /* Copy data from a temporary file to a stream. */
-void
+int
 pdf_copy_data(stream *s, FILE *file, gs_offset_t count, stream_arcfour_state *ss)
 {
     gs_offset_t r, left = count;
@@ -1707,19 +1754,19 @@ pdf_copy_data(stream *s, FILE *file, gs_offset_t count, stream_arcfour_state *ss
 
         r = fread(buf, 1, copy, file);
         if (r < 1) {
-            gs_note_error(gs_error_ioerror);
-            return;
+            return gs_note_error(gs_error_ioerror);
         }
         if (ss)
             s_arcfour_process_buffer(ss, buf, copy);
         stream_write(s, buf, copy);
         left -= copy;
     }
+    return 0;
 }
 
 /* Copy data from a temporary file to a stream,
    which may be targetted to the same file. */
-void
+int
 pdf_copy_data_safe(stream *s, FILE *file, gs_offset_t position, long count)
 {
     long r, left = count;
@@ -1730,22 +1777,20 @@ pdf_copy_data_safe(stream *s, FILE *file, gs_offset_t position, long count)
         int64_t end_pos = gp_ftell_64(file);
 
         if (gp_fseek_64(file, position + count - left, SEEK_SET) != 0) {
-            gs_note_error(gs_error_ioerror);
-            return;
+            return_error(gs_error_ioerror);
         }
         r = fread(buf, 1, copy, file);
         if (r < 1) {
-            gs_note_error(gs_error_ioerror);
-            return;
+            return_error(gs_error_ioerror);
         }
         if (gp_fseek_64(file, end_pos, SEEK_SET) != 0) {
-            gs_note_error(gs_error_ioerror);
-            return;
+            return_error(gs_error_ioerror);
         }
         stream_write(s, buf, copy);
         sflush(s);
         left -= copy;
     }
+    return 0;
 }
 
 /* ------ Pages ------ */
@@ -1764,8 +1809,8 @@ pdf_page_id(gx_device_pdf * pdev, int page_num)
         pdf_page_t *new_pages;
 
         /* Maximum page in PDF is 2^31 - 1. Clamp to that limit here */
-        if (page_num > (1L << 31) - 11)
-            page_num = (1L << 31) - 11;
+        if (page_num > (1LU << 31) - 11)
+            page_num = (1LU << 31) - 11;
         new_num_pages = max(page_num + 10, pdev->num_pages << 1);
 
         new_pages = gs_resize_object(pdev->pdf_memory, pdev->pages, new_num_pages,
@@ -2127,11 +2172,13 @@ pdf_put_composite(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_i
                             if (*p == '/' || *p == '[' || *p == '{' || *p == '(' || *p == ' ') {
                                 stream_write(pdev->strm, start, p - start);
                                 stream_putc(pdev->strm, (byte)'\n');
+                                width = 0;
                                 start = p;
+                                break;
                             }
                             else p--;
                         }
-                        if (p == start) {
+                        if (p == start && width != 0) {
                             stream_write(pdev->strm, start, save - start);
                             stream_putc(pdev->strm, (byte)'\n');
                             start = save;
@@ -2141,6 +2188,12 @@ pdf_put_composite(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_i
                         p++;
                     }
                 }
+                if (width) {
+                    stream_write(pdev->strm, start, p - start);
+                    stream_putc(pdev->strm, (byte)'\n');
+                    width = 0;
+                    start = p;
+                }
             } else {
                 stream_write(pdev->strm, vstr, size);
             }
