@@ -577,6 +577,7 @@ struct gx_device_pdf_s {
     bool ReEncodeCharacters;
     long FirstObjectNumber;
     bool CompressFonts;
+    bool CompressStreams;
     bool PrintStatistics;
     gs_param_string DocumentUUID;
     gs_param_string InstanceUUID;
@@ -600,7 +601,6 @@ struct gx_device_pdf_s {
                               a bitmap representation of a shading.
                               (Bigger shadings to be downsampled). */
     long MaxInlineImageSize;
-    gs_param_int_array DSCEncodingToUnicode;
     /* Encryption parameters */
     gs_param_string OwnerPassword;
     gs_param_string UserPassword;
@@ -850,6 +850,7 @@ struct gx_device_pdf_s {
     bool DetectDuplicateImages;
     bool AllowIncrementalCFF;
     bool WantsToUnicode;
+    bool WantsPageLabels;
     bool AllowPSRepeatFunctions;
     bool IsDistiller;
     bool PreserveSMask;
@@ -859,9 +860,6 @@ struct gx_device_pdf_s {
                                      * This parameter is present only to allow
                                      * ps2write output to work on those pritners.
                                      */
-    bool UseOldColor;               /* Use the old pdfwrite colour conversions instead of the CMS
-                                     * temporary variable
-                                     */
     bool Linearise;                 /* Whether to Linearizse the file, the next 2 parameter
                                      * are only used if this is true.
                                      */
@@ -881,6 +879,16 @@ struct gx_device_pdf_s {
                                      */
     bool FlattenFonts;
     int LastFormID;
+    char *ExtensionMetadata;        /* If present the non-standard pdfmark Extension_Metadata has been
+                                     * used, the content of ths string is written out as part of the
+                                     * metadata referenced by the Catalog.
+                                     */
+    char *PDFFormName;              /* If present, we are processing (or about to process) a Form XObject
+                                     * which we wish to handle as a Form XObject, not flatten. Currently
+                                     * this is only the case for Annotation Appearances. This must be NULL
+                                     * after the form is processed. The name will be used to create a
+                                     * local named object which pdfmark can reference.
+                                     */
 };
 
 #define is_in_page(pdev)\
@@ -908,13 +916,14 @@ struct gx_device_pdf_s {
  m(28,sbstack) m(29,substream_Resources) m(30,font3)\
  m(31,accumulating_substream_resource) \
  m(32,pres_soft_mask_dict) m(33,PDFXTrimBoxToMediaBoxOffset.data)\
- m(34,PDFXBleedBoxToTrimBoxOffset.data) m(35, DSCEncodingToUnicode.data)\
- m(36,Identity_ToUnicode_CMaps[0]) m(37,Identity_ToUnicode_CMaps[1])\
- m(38,ResourceUsage) m(39,vgstack)\
- m(40, outline_levels) m(41, EmbeddedFiles)
- m(41, gx_device_pdf, EmbeddedFiles);
- m(42, gx_device_pdf, pdf_font_dir);*/
-#define gx_device_pdf_num_ptrs 43
+ m(34,PDFXBleedBoxToTrimBoxOffset.data)
+ m(35,Identity_ToUnicode_CMaps[0]) m(36,Identity_ToUnicode_CMaps[1])\
+ m(37,vgstack)\
+ m(38, outline_levels)
+ m(39, gx_device_pdf, EmbeddedFiles);
+ m(40, gx_device_pdf, pdf_font_dir);
+ m(41, gx_device_pdf, Extension_Metadata);*/
+#define gx_device_pdf_num_ptrs 42
 #define gx_device_pdf_do_param_strings(m)\
     m(0, OwnerPassword) m(1, UserPassword) m(2, NoEncrypt)\
     m(3, DocumentUUID) m(4, InstanceUUID)
@@ -1071,6 +1080,13 @@ pdf_resource_t *pdf_find_resource_by_gs_id(gx_device_pdf * pdev,
                                            pdf_resource_type_t rtype,
                                            gs_id rid);
 
+/* Remove a resource from a chain of resources but do not free it. The resource will
+ * have to be listed elsewhere. This is primarily useful for moving existing resources
+ * to local named resources.
+ */
+void
+pdf_drop_resource_from_chain(gx_device_pdf * pdev, pdf_resource_t *pres1, pdf_resource_type_t rtype);
+
 void pdf_drop_resources(gx_device_pdf * pdev, pdf_resource_type_t rtype,
         int (*cond)(gx_device_pdf * pdev, pdf_resource_t *pres));
 
@@ -1127,8 +1143,8 @@ int pdf_free_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype);
 int pdf_store_page_resources(gx_device_pdf *pdev, pdf_page_t *page, bool clear_usage);
 
 /* Copy data from a temporary file to a stream. */
-void pdf_copy_data(stream *s, FILE *file, gs_offset_t count, stream_arcfour_state *ss);
-void pdf_copy_data_safe(stream *s, FILE *file, gs_offset_t position, long count);
+int pdf_copy_data(stream *s, FILE *file, gs_offset_t count, stream_arcfour_state *ss);
+int pdf_copy_data_safe(stream *s, FILE *file, gs_offset_t position, long count);
 
 /* Add the encryption filter. */
 int pdf_begin_encrypt(gx_device_pdf * pdev, stream **s, gs_id object_id);
@@ -1163,7 +1179,7 @@ int pdf_write_saved_string(gx_device_pdf * pdev, gs_string * pstr);
 /* Store a copy of clipping path. */
 int pdf_remember_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath);
 
-int pdf_check_soft_mask(gx_device_pdf * pdev, gs_imager_state * pis);
+int pdf_check_soft_mask(gx_device_pdf * pdev, gs_gstate * pgs);
 
 /* Test whether the clip path needs updating. */
 bool pdf_must_put_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath);
@@ -1344,8 +1360,8 @@ int pdfmark_write_article(gx_device_pdf * pdev, const pdf_article_t * part);
 bool pdf_objname_is_valid(const byte *data, uint size);
 
 /*
- * Look up a named object.  Return e_rangecheck if the syntax is invalid,
- * e_undefined if no object by that name exists.
+ * Look up a named object.  Return_error(gs_error_rangecheck if the syntax is invalid,
+ * gs_error_undefined if no object by that name exists.
  */
 int pdf_find_named(gx_device_pdf * pdev, const gs_param_string * pname,
                    cos_object_t **ppco);
@@ -1370,7 +1386,7 @@ int pdf_refer_named(gx_device_pdf *pdev, const gs_param_string *pname,
 
 /*
  * Look up a named object as for pdf_refer_named.  If the object already
- * exists and is not simply a forward reference, return e_rangecheck;
+ * exists and is not simply a forward reference, return_error(gs_error_rangecheck);
  * if it exists as a forward reference, set its type and return 0;
  * otherwise, create the object with the given type and return 1.
  * pname = 0 is allowed: in this case, simply create the object.
@@ -1382,8 +1398,8 @@ int pdf_make_named_dict(gx_device_pdf * pdev, const gs_param_string * pname,
 
 /*
  * Look up a named object as for pdf_refer_named.  If the object does not
- * exist, or is a forward reference, return e_undefined; if the object
- * exists has the wrong type, return e_typecheck.
+ * exist, or is a forward reference, return gs_error_undefined; if the object
+ * exists has the wrong type, return gs_error_typecheck.
  */
 int pdf_get_named(gx_device_pdf * pdev, const gs_param_string * pname,
                   cos_type_t cotype, cos_object_t **ppco);
@@ -1437,7 +1453,6 @@ void pdf_text_state_copy(pdf_text_state_t *pts_to, pdf_text_state_t *pts_from);
 void pdf_reset_text_page(pdf_text_data_t *ptd);
 void pdf_reset_text_state(pdf_text_data_t *ptd);
 void pdf_close_text_page(gx_device_pdf *pdev);
-int  pdf_get_stoted_text_size(pdf_text_state_t *state);
 
 /* For gdevpdfb.c */
 
