@@ -91,14 +91,14 @@ pixel_resize(psdf_binary_writer * pbw, int width, int num_components,
 }
 
 static int
-convert_color(gx_device *pdev, const gs_color_space *pcs, const gs_imager_state * pis,
+convert_color(gx_device *pdev, const gs_color_space *pcs, const gs_gstate * pgs,
               gs_client_color *cc, float c[3])
 {
     int code;
     gx_device_color dc;
 
     cs_restrict_color(cc, pcs);
-    code = pcs->type->remap_color(cc, pcs, &dc, pis, pdev, gs_color_select_texture);
+    code = pcs->type->remap_color(cc, pcs, &dc, pgs, pdev, gs_color_select_texture);
     if (code < 0)
         return code;
     c[0] = (float)((int)(dc.colors.pure >> pdev->color_info.comp_shift[0]) & ((1 << pdev->color_info.comp_bits[0]) - 1));
@@ -110,7 +110,7 @@ convert_color(gx_device *pdev, const gs_color_space *pcs, const gs_imager_state
 /* A heuristic choice of DCT compression parameters - see bug 687174. */
 static int
 choose_DCT_params(gx_device *pdev, const gs_color_space *pcs,
-                  const gs_imager_state * pis,
+                  const gs_gstate * pgs,
                   gs_c_param_list *list, gs_c_param_list **param,
                   stream_state *st)
 {
@@ -139,25 +139,25 @@ choose_DCT_params(gx_device *pdev, const gs_color_space *pcs,
     mdev.color_info.separable_and_linear = GX_CINFO_SEP_LIN;
     /* Set mem device icc profile */
     gsicc_init_device_profile_struct((gx_device *) &mdev, NULL, 0);
-    if (pis) {
+    if (pgs) {
         /* Check for an RGB-like color space.
            To recognize that we make a matrix as it were a linear operator,
            suppress an ununiformity by subtracting the image of {0,0,0},
            and then check for giagonal domination.  */
         cc.paint.values[0] = cc.paint.values[1] = cc.paint.values[2] = MIN_FLOAT;
-        code = convert_color((gx_device *)&mdev, pcs, pis, &cc, c[3]);
+        code = convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[3]);
         if (code < 0)
             return code;
         cc.paint.values[0] = MAX_FLOAT; cc.paint.values[1] = MIN_FLOAT; cc.paint.values[2] = MIN_FLOAT;
-        code = convert_color((gx_device *)&mdev, pcs, pis, &cc, c[0]);
+        code = convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[0]);
         if (code < 0)
             return code;
         cc.paint.values[0] = MIN_FLOAT; cc.paint.values[1] = MAX_FLOAT; cc.paint.values[2] = MIN_FLOAT;
-        code = convert_color((gx_device *)&mdev, pcs, pis, &cc, c[1]);
+        code = convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[1]);
         if (code < 0)
             return code;
         cc.paint.values[0] = MIN_FLOAT; cc.paint.values[1] = MIN_FLOAT; cc.paint.values[2] = MAX_FLOAT;
-        code = convert_color((gx_device *)&mdev, pcs, pis, &cc, c[2]);
+        code = convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[2]);
         if (code < 0)
             return code;
         c[0][0] -= c[3][0]; c[0][1] -= c[3][1]; c[0][2] -= c[3][2];
@@ -180,11 +180,11 @@ choose_DCT_params(gx_device *pdev, const gs_color_space *pcs,
         /* Check for a Lab-like color space.
            Colors {v,0,0} should map to grays. */
         cc.paint.values[0] = MAX_FLOAT; cc.paint.values[1] = cc.paint.values[2] = 0;
-        convert_color((gx_device *)&mdev, pcs, pis, &cc, c[0]);
+        convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[0]);
         cc.paint.values[0] /= 2;
-        convert_color((gx_device *)&mdev, pcs, pis, &cc, c[1]);
+        convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[1]);
         cc.paint.values[0] /= 2;
-        convert_color((gx_device *)&mdev, pcs, pis, &cc, c[2]);
+        convert_color((gx_device *)&mdev, pcs, pgs, &cc, c[2]);
         c[0][1] -= c[0][0]; c[0][2] -= c[0][0];
         c[1][1] -= c[1][0]; c[1][2] -= c[1][0];
         c[2][1] -= c[2][0]; c[2][2] -= c[2][0];
@@ -192,7 +192,7 @@ choose_DCT_params(gx_device *pdev, const gs_color_space *pcs,
         c[1][1] = any_abs(c[1][1]); c[1][2] = any_abs(c[1][2]);
         c[2][1] = any_abs(c[2][1]); c[2][2] = any_abs(c[2][2]);
     }
-    if (pis && c[0][0] * domination > c[0][1] && c[0][0] * domination > c[0][2] &&
+    if (pgs && c[0][0] * domination > c[0][1] && c[0][0] * domination > c[0][2] &&
         c[1][0] * domination > c[1][1] && c[1][0] * domination > c[1][2] &&
         c[2][0] * domination > c[2][1] && c[2][0] * domination > c[2][2]) {
         /* Yes, it looks like an Lab color space.
@@ -228,7 +228,7 @@ error:
 /* Add the appropriate image compression filter, if any. */
 static int
 setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
-                        const gs_pixel_image_t * pim, const gs_imager_state * pis,
+                        const gs_pixel_image_t * pim, const gs_gstate * pgs,
                         bool lossless)
 {
     gx_device_psdf *pdev = pbw->dev;
@@ -271,7 +271,7 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
         }
         dict = pdip->ACSDict;
     } else if (!lossless)
-        return gs_error_rangecheck; /* Reject the alternative stream. */
+      return_error(gs_error_rangecheck); /* Reject the alternative stream. */
     if (pdev->version < psdf_version_ll3 && templat == &s_zlibE_template)
         templat = lossless_template;
     if (dict != NULL) /* Some interpreters don't supply filter parameters. */
@@ -341,7 +341,7 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
         gs_c_param_list list, *param = dict;
 
         gs_c_param_list_write(&list, mem);
-        code = choose_DCT_params((gx_device *)pbw->dev, pcs, pis, &list, &param, st);
+        code = choose_DCT_params((gx_device *)pbw->dev, pcs, pgs, &list, &param, st);
         if (code < 0) {
             gs_c_param_list_release(&list);
             return code;
@@ -413,7 +413,7 @@ do_downsample(const psdf_image_params *pdip, const gs_pixel_image_t *pim,
 /* Assumes do_downsampling() is true. */
 static int
 setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
-                   gs_pixel_image_t * pim, const gs_imager_state * pis,
+                   gs_pixel_image_t * pim, const gs_gstate * pgs,
                    double resolution, bool lossless)
 {
     gx_device_psdf *pdev = pbw->dev;
@@ -425,34 +425,44 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
     stream_state *st;
     int code;
 
-    switch (pdip->DownsampleType) {
-        case ds_Subsample:
-            templat = &s_Subsample_template;
-            break;
-        case ds_Average:
-            templat = &s_Average_template;
-            break;
-        case ds_Bicubic:
-            templat = &s_IScale_template;
-            /* We now use the Mitchell filter instead of the 'bicubic' filter
-             * because it gives better results.
-            templat = &s_Bicubic_template;
-             */
-            break;
-        default:
-            dmprintf1(pdev->v_memory, "Unsupported downsample type %d\n", pdip->DownsampleType);
-            return gs_note_error(gs_error_rangecheck);
-    }
+    /* We can't apply anything other than a simple downsample to monochrome
+     * image without turning them into greyscale images. We set the default
+     * to subsample above, so just ignore it if the current image is monochtome.
+     */
+    if (pim->BitsPerComponent > 1) {
+        switch (pdip->DownsampleType) {
+            case ds_Subsample:
+                templat = &s_Subsample_template;
+                break;
+            case ds_Average:
+                templat = &s_Average_template;
+                break;
+            case ds_Bicubic:
+                templat = &s_IScale_template;
+                /* We now use the Mitchell filter instead of the 'bicubic' filter
+                 * because it gives better results.
+                templat = &s_Bicubic_template;
+                 */
+                break;
+            default:
+                dmprintf1(pdev->v_memory, "Unsupported downsample type %d\n", pdip->DownsampleType);
+                return gs_note_error(gs_error_rangecheck);
+        }
 
-    if (pdip->DownsampleType != ds_Bicubic) {
-        /* If downsample type is not bicubic, ensure downsample factor is
-         * an integer if we're close to one (< 0.1) or silently switch to
-         * bicubic transform otherwise. See bug #693917. */
-        float rfactor = floor(factor + 0.5);
-        if (fabs(rfactor-factor) < 0.1)
-            factor = rfactor;  /* round factor to nearest integer */
-        else
-            templat = &s_Bicubic_template;  /* switch to bicubic */
+        if (pdip->DownsampleType != ds_Bicubic) {
+            /* If downsample type is not bicubic, ensure downsample factor is
+             * an integer if we're close to one (< 0.1) or silently switch to
+             * bicubic transform otherwise. See bug #693917. */
+            float rfactor = floor(factor + 0.5);
+            if (fabs(rfactor-factor) < 0.1)
+                factor = rfactor;  /* round factor to nearest integer */
+            else
+                templat = &s_Bicubic_template;  /* switch to bicubic */
+        }
+    } else {
+        if (pdip->DownsampleType != ds_Subsample) {
+            dmprintf(pdev->memory, "The only Downsample filter for monochrome images is Subsample, ignoring request.\n");
+        }
     }
 
     st = s_alloc_state(pdev->v_memory, templat->stype,
@@ -475,8 +485,18 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
         ss->AntiAlias = pdip->AntiAlias;
         ss->padX = ss->padY = false; /* should be true */
 
-        if (templat->init)
-            templat->init(st);
+        if (pim->BitsPerComponent == 1) {
+            if (floor(ss->XFactor) != ss->XFactor)
+                factor = ss->YFactor = ss->XFactor = floor(ss->XFactor + 0.5);
+        }
+        if (templat->init) {
+            code = templat->init(st);
+            if (code < 0) {
+                dmprintf(st->memory, "Failed to initialise downsample filter, downsampling aborted\n");
+                gs_free_object(pdev->v_memory, st, "setup_image_compression");
+                return 0;
+            }
+        }
         pim->BitsPerComponent = pdip->Depth;
         pim->Width = s_Downsample_size_out(pim->Width, factor, false);
         pim->Height = s_Downsample_size_out(pim->Height, factor, false);
@@ -484,7 +504,7 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
                         (double)pim->Height / orig_height,
                         &pim->ImageMatrix);
         /****** NO ANTI-ALIASING YET ******/
-        if ((code = setup_image_compression(pbw, pdip, pim, pis, lossless)) < 0 ||
+        if ((code = setup_image_compression(pbw, pdip, pim, pgs, lossless)) < 0 ||
             (code = pixel_resize(pbw, pim->Width, ss->Colors,
                                  8, pdip->Depth)) < 0 ||
             (code = psdf_encode_binary(pbw, templat, st)) < 0 ||
@@ -514,8 +534,14 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
         ss->params.early_cm = true;
         ss->params.MaxValueIn = ss->params.MaxValueOut = (int)pow(2, pdip->Depth);;
 
-        if (templat->init)
-            templat->init(st);
+        if (templat->init) {
+            code = templat->init(st);
+            if (code < 0) {
+                dmprintf(st->memory, "Failed to initialise downsample filter, downsampling aborted\n");
+                gs_free_object(pdev->v_memory, st, "setup_image_compression");
+                return 0;
+            }
+        }
         pim->Width = s_Downsample_size_out(pim->Width, factor, false);
         pim->Height = s_Downsample_size_out(pim->Height, factor, false);
         pim->BitsPerComponent = pdip->Depth;
@@ -523,7 +549,7 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
                         (double)pim->Height / orig_height,
                         &pim->ImageMatrix);
         /****** NO ANTI-ALIASING YET ******/
-        if ((code = setup_image_compression(pbw, pdip, pim, pis, lossless)) < 0 ||
+        if ((code = setup_image_compression(pbw, pdip, pim, pgs, lossless)) < 0 ||
             (code = pixel_resize(pbw, pim->Width, Colors,
                                  8, pdip->Depth)) < 0 ||
             (code = psdf_encode_binary(pbw, templat, st)) < 0 ||
@@ -540,10 +566,10 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
 /* Decive whether to convert an image to RGB. */
 bool
 psdf_is_converting_image_to_RGB(const gx_device_psdf * pdev,
-                const gs_imager_state * pis, const gs_pixel_image_t * pim)
+                const gs_gstate * pgs, const gs_pixel_image_t * pim)
 {
     return pdev->params.ConvertCMYKImagesToRGB &&
-            pis != 0 && pim->ColorSpace &&
+            pgs != 0 && pim->ColorSpace &&
             (gs_color_space_get_index(pim->ColorSpace) == gs_color_space_index_DeviceCMYK ||
             (gs_color_space_get_index(pim->ColorSpace) == gs_color_space_index_ICC
             && gsicc_get_default_type(pim->ColorSpace->cmm_icc_profile_data) ==
@@ -588,7 +614,7 @@ adjust_auto_filter_strategy_mono(gx_device_psdf *pdev,
 int
 psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
                          gs_pixel_image_t * pim, const gs_matrix * pctm,
-                         const gs_imager_state * pis, bool lossless, bool in_line)
+                         const gs_gstate * pgs, bool lossless, bool in_line)
 {
     /*
      * The following algorithms are per Adobe Tech Note # 5151,
@@ -671,17 +697,17 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
                 params.Dict = pdev->params.GrayImage.Dict;
                 adjust_auto_filter_strategy(pdev, &params, pdev->params.GrayImage.Dict, pim, in_line);
             }
-            code = setup_downsampling(pbw, &params, pim, pis, resolution, lossless);
+            code = setup_downsampling(pbw, &params, pim, pgs, resolution, lossless);
         } else {
             adjust_auto_filter_strategy(pdev, &params, pdev->params.GrayImage.Dict, pim, in_line);
-            code = setup_image_compression(pbw, &params, pim, pis, lossless);
+            code = setup_image_compression(pbw, &params, pim, pgs, lossless);
         }
         if (code < 0)
             return code;
         code = pixel_resize(pbw, pim->Width, ncomp, bpc, bpc_out);
     } else {
         /* Color */
-        bool cmyk_to_rgb = psdf_is_converting_image_to_RGB(pdev, pis, pim);
+        bool cmyk_to_rgb = psdf_is_converting_image_to_RGB(pdev, pgs, pim);
 
         if (cmyk_to_rgb) {
             gs_memory_t *mem = pdev->v_memory;
@@ -689,15 +715,17 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
             /* {csrc} decref old colorspace? */
             rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
             pim->ColorSpace = gs_cspace_new_DeviceRGB(mem);
+            if (pim->ColorSpace == NULL)
+                return_error(gs_error_VMerror);
         }
         if (params.Depth == -1)
             params.Depth = (cmyk_to_rgb ? 8 : bpc_out);
         if (do_downsample(&params, pim, resolution)) {
             adjust_auto_filter_strategy(pdev, &params, pdev->params.ColorImage.Dict, pim, in_line);
-            code = setup_downsampling(pbw, &params, pim, pis, resolution, lossless);
+            code = setup_downsampling(pbw, &params, pim, pgs, resolution, lossless);
         } else {
             adjust_auto_filter_strategy(pdev, &params, pdev->params.ColorImage.Dict, pim, in_line);
-            code = setup_image_compression(pbw, &params, pim, pis, lossless);
+            code = setup_image_compression(pbw, &params, pim, pgs, lossless);
         }
         if (code < 0)
             return code;
@@ -713,7 +741,7 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
                 (code = pixel_resize(pbw, pim->Width, 4, bpc, 8)) < 0
                 )
                 return code;
-            s_C2R_init(ss, pis);
+            s_C2R_init(ss, pgs);
         } else {
             code = pixel_resize(pbw, pim->Width, ncomp, bpc, bpc_out);
             if (code < 0)
@@ -723,7 +751,7 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
     return code;
 }
 
-/* Set up compression filters for a lossless image, with no downsampling, */
+/* Set up compression filters for a lossless image, downsampling is permitted, */
 /* no color space conversion, and only lossless filters. */
 /* Note that this may modify the image parameters. */
 int
@@ -738,12 +766,10 @@ psdf_setup_lossless_filters(gx_device_psdf *pdev, psdf_binary_writer *pbw,
 
     ipdev = *pdev;
     ipdev.params.ColorImage.AutoFilter = false;
-    ipdev.params.ColorImage.Downsample = false;
     ipdev.params.ColorImage.Filter = "FlateEncode";
     ipdev.params.ColorImage.filter_template = &s_zlibE_template;
     ipdev.params.ConvertCMYKImagesToRGB = false;
     ipdev.params.GrayImage.AutoFilter = false;
-    ipdev.params.GrayImage.Downsample = false;
     ipdev.params.GrayImage.Filter = "FlateEncode";
     ipdev.params.GrayImage.filter_template = &s_zlibE_template;
     return psdf_setup_image_filters(&ipdev, pbw, pim, NULL, NULL, true, in_line);
@@ -801,12 +827,12 @@ psdf_setup_image_to_mask_filter(psdf_binary_writer *pbw, gx_device_psdf *pdev,
 int
 psdf_setup_image_colors_filter(psdf_binary_writer *pbw,
                                gx_device_psdf *pdev, gs_pixel_image_t * pim,
-                               const gs_imager_state *pis)
+                               const gs_gstate *pgs)
 {   /* fixme: currently it's a stub convertion to mask. */
     int code;
     stream_state *ss = s_alloc_state(pdev->memory, s__image_colors_template.stype,
         "psdf_setup_image_colors_filter");
-    int i;
+    unsigned char i;
 
     if (ss == 0)
         return_error(gs_error_VMerror);
@@ -820,7 +846,7 @@ psdf_setup_image_colors_filter(psdf_binary_writer *pbw,
                     gs_color_space_num_components(pim->ColorSpace),
                     pim->BitsPerComponent);
     s_image_colors_set_color_space((stream_image_colors_state *)ss,
-                    (gx_device *)pdev, pim->ColorSpace, pis, pim->Decode);
+                    (gx_device *)pdev, pim->ColorSpace, pgs, pim->Decode);
     pim->BitsPerComponent = pdev->color_info.comp_bits[0]; /* Same precision for all components. */
     for (i = 0; i < pdev->color_info.num_components; i++) {
         pim->Decode[i * 2 + 0] = 0;
@@ -834,7 +860,7 @@ psdf_setup_image_colors_filter(psdf_binary_writer *pbw,
 int
 new_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
                          gs_pixel_image_t * pim, const gs_matrix * pctm,
-                         const gs_imager_state * pis, bool lossless, bool in_line,
+                         const gs_gstate * pgs, bool lossless, bool in_line,
                          bool colour_conversion)
 {
     /*
@@ -931,10 +957,10 @@ new_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
                 params.Dict = pdev->params.GrayImage.Dict;
                 adjust_auto_filter_strategy(pdev, &params, pdev->params.GrayImage.Dict, pim, in_line);
             }
-            code = setup_downsampling(pbw, &params, pim, pis, resolution, lossless);
+            code = setup_downsampling(pbw, &params, pim, pgs, resolution, lossless);
         } else {
             adjust_auto_filter_strategy(pdev, &params, pdev->params.GrayImage.Dict, pim, in_line);
-            code = setup_image_compression(pbw, &params, pim, pis, lossless);
+            code = setup_image_compression(pbw, &params, pim, pgs, lossless);
         }
         if (code < 0)
             return code;
@@ -945,10 +971,10 @@ new_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
             params.Depth = (colour_conversion ? 8 : bpc_out);
         if (do_downsample(&params, pim, resolution)) {
             adjust_auto_filter_strategy(pdev, &params, pdev->params.ColorImage.Dict, pim, in_line);
-            code = setup_downsampling(pbw, &params, pim, pis, resolution, lossless);
+            code = setup_downsampling(pbw, &params, pim, pgs, resolution, lossless);
         } else {
             adjust_auto_filter_strategy(pdev, &params, pdev->params.ColorImage.Dict, pim, in_line);
-            code = setup_image_compression(pbw, &params, pim, pis, lossless);
+            code = setup_image_compression(pbw, &params, pim, pgs, lossless);
         }
         if (code < 0)
             return code;
@@ -962,17 +988,16 @@ new_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 int
 new_setup_lossless_filters(gx_device_psdf *pdev, psdf_binary_writer *pbw,
                             gs_pixel_image_t *pim, bool in_line,
-                            bool colour_conversion)
+                            bool colour_conversion, const gs_matrix *pctm, gs_gstate * pgs)
 {
     /*
      * Set up a device with modified parameters for computing the image
-     * compression filters.  Don't allow downsampling or lossy compression.
+     * compression filters.  Don't allow lossy compression, but do allow downsampling.
      */
     gx_device_psdf ipdev;
 
     ipdev = *pdev;
     ipdev.params.ColorImage.AutoFilter = false;
-    ipdev.params.ColorImage.Downsample = false;
     ipdev.params.ColorImage.Filter = "FlateEncode";
     ipdev.params.ColorImage.filter_template = &s_zlibE_template;
     ipdev.params.ConvertCMYKImagesToRGB = false;
@@ -980,7 +1005,7 @@ new_setup_lossless_filters(gx_device_psdf *pdev, psdf_binary_writer *pbw,
     ipdev.params.GrayImage.Downsample = false;
     ipdev.params.GrayImage.Filter = "FlateEncode";
     ipdev.params.GrayImage.filter_template = &s_zlibE_template;
-    return new_setup_image_filters(&ipdev, pbw, pim, NULL, NULL, true, in_line, colour_conversion);
+    return new_setup_image_filters(&ipdev, pbw, pim, pctm, pgs, true, in_line, colour_conversion);
 }
 
 int new_resize_input(psdf_binary_writer *pbw, int width, int num_comps, int bpc_in, int bpc_out)
