@@ -36,7 +36,7 @@
 #include "zicc.h"
 #include "gsicc_manage.h"
 #include "gx.h"
-#include "gxistate.h"
+#include "gxgstate.h"
 #include "gsicc_create.h"
 #include "gsicc_profilecache.h"
 #include "gxdevice.h"  /* for output intent setting */
@@ -49,7 +49,6 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
     ref *                   pstrmval;
     stream *                s = 0L;
     cmm_profile_t           *picc_profile = NULL;
-    gs_imager_state *       pis = (gs_imager_state *)igs;
     int                     i, expected = 0;
     ref *                   pnameval;
     static const char *const icc_std_profile_names[] = {
@@ -65,7 +64,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
     check_read_file(i_ctx_p, s, pstrmval);
 
     /* build the color space object */
-    code = gs_cspace_build_ICC(&pcs, NULL, gs_state_memory(igs));
+    code = gs_cspace_build_ICC(&pcs, NULL, gs_gstate_memory(igs));
     if (code < 0)
         return gs_rethrow(code, "building color space object");
     /*  For now, dump the profile into a buffer
@@ -79,7 +78,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
        specs that have enumerated types to indicate sRGB sGray etc */
     if (dict_find_string(ICCdict, "Name", &pnameval) > 0){
         uint size = r_size(pnameval);
-        char *str = (char *)gs_alloc_bytes(gs_state_memory(igs), size+1, "seticc");
+        char *str = (char *)gs_alloc_bytes(gs_gstate_memory(igs), size+1, "seticc");
         memcpy(str, (const char *)pnameval->value.bytes, size);
         str[size] = 0;
 
@@ -87,18 +86,20 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
         for (k = 0; k < GSICC_NUMBER_STANDARD_PROFILES; k++) {
             if ( strcmp( str, icc_std_profile_keys[k] ) == 0 ) {
                 picc_profile = gsicc_get_profile_handle_file(icc_std_profile_names[k],
-                    strlen(icc_std_profile_names[k]), gs_state_memory(igs));
+                    strlen(icc_std_profile_names[k]), gs_gstate_memory(igs));
                 break;
             }
         }
     } else {
-        picc_profile = gsicc_profile_new(s, gs_state_memory(igs), NULL, 0);
+        picc_profile = gsicc_profile_new(s, gs_gstate_memory(igs), NULL, 0);
+        if (picc_profile == NULL)
+            return gs_throw(gs_error_VMerror, "Creation of ICC profile failed");
         /* We have to get the profile handle due to the fact that we need to know
            if it has a data space that is CIELAB */
         picc_profile->profile_handle =
             gsicc_get_profile_handle_buffer(picc_profile->buffer,
                                             picc_profile->buffer_size,
-                                            gs_state_memory(igs));
+                                            gs_gstate_memory(igs));
     }
     if (picc_profile == NULL || picc_profile->profile_handle == NULL) {
         /* Free up everything, the profile is not valid. We will end up going
@@ -107,7 +108,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
         rc_decrement(pcs,"seticc");
         return -1;
     }
-    code = gsicc_set_gscs_profile(pcs, picc_profile, gs_state_memory(igs));
+    code = gsicc_set_gscs_profile(pcs, picc_profile, gs_gstate_memory(igs));
     if (code < 0) {
         rc_decrement(picc_profile,"seticc");
         rc_decrement(pcs,"seticc");
@@ -141,7 +142,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
 
     /* Lets go ahead and get the hash code and check if we match one of the default spaces */
     /* Later we may want to delay this, but for now lets go ahead and do it */
-    gsicc_init_hash_cs(picc_profile, pis);
+    gsicc_init_hash_cs(picc_profile, igs);
 
     /* Set the range according to the data type that is associated with the
        ICC input color type.  Occasionally, we will run into CIELAB to CIELAB
@@ -177,26 +178,26 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
     }
     /* Now see if we are in an overide situation.  We have to wait until now
        in case this is an LAB profile which we will not overide */
-    if (gs_currentoverrideicc(pis) && picc_profile->data_cs != gsCIELAB) {
+    if (gs_currentoverrideicc(igs) && picc_profile->data_cs != gsCIELAB) {
         /* Free up the profile structure */
         switch( picc_profile->data_cs ) {
             case gsRGB:
-                pcs->cmm_icc_profile_data = pis->icc_manager->default_rgb;
+                pcs->cmm_icc_profile_data = igs->icc_manager->default_rgb;
                 break;
             case gsGRAY:
-                pcs->cmm_icc_profile_data = pis->icc_manager->default_gray;
+                pcs->cmm_icc_profile_data = igs->icc_manager->default_gray;
                 break;
             case gsCMYK:
-                pcs->cmm_icc_profile_data = pis->icc_manager->default_cmyk;
+                pcs->cmm_icc_profile_data = igs->icc_manager->default_cmyk;
                 break;
             default:
                 break;
         }
-        /* Have one increment from the color space.  Having these tied 
+        /* Have one increment from the color space.  Having these tied
            together is not really correct.  Need to fix that.  ToDo.  MJV */
         rc_adjust(picc_profile, -2, "seticc");
         rc_increment(pcs->cmm_icc_profile_data);
-    } 
+    }
     /* Set the color space.  We are done.  No joint cache here... */
     code = gs_setcolorspace(igs, pcs);
     /* The context has taken a reference to the colorspace. We no longer need
@@ -214,7 +215,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
 /*
  *   <dict>  .set_outputintent  -
  *
- * Set and use the specified output intent.  
+ * Set and use the specified output intent.
  *
  */
 static int
@@ -223,7 +224,6 @@ zset_outputintent(i_ctx_t * i_ctx_p)
     os_ptr                  op = osp;
     int                     code = 0;
     gx_device *dev = gs_currentdevice(igs);
-    gs_imager_state *pis = (gs_imager_state *)igs;
     cmm_dev_profile_t       *dev_profile;
     stream *                s = 0L;
     ref *                   pnval;
@@ -232,7 +232,7 @@ zset_outputintent(i_ctx_t * i_ctx_p)
     cmm_profile_t           *picc_profile;
     int                     expected = 0;
     gs_color_space_index    index;
-    gsicc_manager_t         *icc_manager = pis->icc_manager;
+    gsicc_manager_t         *icc_manager = igs->icc_manager;
     cmm_profile_t           *source_profile = NULL;
 
     check_type(*op, t_dictionary);
@@ -241,9 +241,13 @@ zset_outputintent(i_ctx_t * i_ctx_p)
 
     /* Get the device structure */
     code = dev_proc(dev, get_profile)(dev,  &dev_profile);
-    if (dev_profile == NULL) { 
+    if (dev_profile == NULL) {
         code = gsicc_init_device_profile_struct(dev, NULL, 0);
+        if (code < 0)
+            return code;
         code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+        if (code < 0)
+            return code;
     }
     if (dev_profile->oi_profile != NULL) {
         return 0;  /* Allow only one setting of this object */
@@ -258,12 +262,14 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         return_error(gs_error_undefined);
     check_read_file(i_ctx_p, s, pstrmval);
 
-    picc_profile = gsicc_profile_new(s, gs_state_memory(igs), NULL, 0);
+    picc_profile = gsicc_profile_new(s, gs_gstate_memory(igs), NULL, 0);
+    if (picc_profile == NULL)
+        return gs_throw(gs_error_VMerror, "Creation of ICC profile failed");
     picc_profile->num_comps = ncomps;
     picc_profile->profile_handle =
         gsicc_get_profile_handle_buffer(picc_profile->buffer,
                                         picc_profile->buffer_size,
-                                        gs_state_memory(igs));
+                                        gs_gstate_memory(igs));
     if (picc_profile->profile_handle == NULL) {
         rc_decrement(picc_profile,"zset_outputintent");
         return -1;
@@ -287,33 +293,33 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         case gsNCHANNEL:
             expected = 0;
             break;
-        case gsNAMED:            
-        case gsUNDEFINED:        
+        case gsNAMED:
+        case gsUNDEFINED:
             break;
     }
     if (expected && ncomps != expected) {
         rc_decrement(picc_profile,"zset_outputintent");
         return_error(gs_error_rangecheck);
     }
-    gsicc_init_hash_cs(picc_profile, pis);
+    gsicc_init_hash_cs(picc_profile, igs);
 
     /* All is well with the profile.  Lets set the stuff that needs to be set */
     dev_profile->oi_profile = picc_profile;
-    picc_profile->name = (char *) gs_alloc_bytes(picc_profile->memory, 
+    picc_profile->name = (char *) gs_alloc_bytes(picc_profile->memory,
                                                  MAX_DEFAULT_ICC_LENGTH,
                                                  "zset_outputintent");
     strncpy(picc_profile->name, OI_PROFILE, strlen(OI_PROFILE));
     picc_profile->name[strlen(OI_PROFILE)] = 0;
     picc_profile->name_length = strlen(OI_PROFILE);
     /* Set the range of the profile */
-    gscms_set_icc_range(&picc_profile);
+    gsicc_set_icc_range(&picc_profile);
 
-    /* If the output device has a different number of componenets, then we are 
-       going to set the output intent as the proofing profile, unless the 
-       proofing profile has already been set. 
+    /* If the output device has a different number of componenets, then we are
+       going to set the output intent as the proofing profile, unless the
+       proofing profile has already been set.
 
-       If the device has the same number of components (and color model) then as 
-       the profile we will use this as the output profile, unless someone has 
+       If the device has the same number of components (and color model) then as
+       the profile we will use this as the output profile, unless someone has
        explicitly set the output profile.
 
        Finally, we will use the output intent profile for the default profile
@@ -323,22 +329,22 @@ zset_outputintent(i_ctx_t * i_ctx_p)
     dev_comps = dev_profile->device_profile[0]->num_comps;
     index = gsicc_get_default_type(dev_profile->device_profile[0]);
     if (ncomps == dev_comps && index < gs_color_space_index_DevicePixel) {
-        /* The OI profile is the same type as the profile for the device and a 
-           "default" profile for the device was not externally set. So we go 
-           ahead and use the OI profile as the device profile.  Care needs to be 
-           taken here to keep from screwing up any device parameters.   We will 
-           use a keyword of OIProfile for the user/device parameter to indicate 
-           its usage.  Also, note conflicts if one is setting object dependent 
+        /* The OI profile is the same type as the profile for the device and a
+           "default" profile for the device was not externally set. So we go
+           ahead and use the OI profile as the device profile.  Care needs to be
+           taken here to keep from screwing up any device parameters.   We will
+           use a keyword of OIProfile for the user/device parameter to indicate
+           its usage.  Also, note conflicts if one is setting object dependent
            color management */
-        rc_assign(dev_profile->device_profile[0], picc_profile, 
+        rc_assign(dev_profile->device_profile[0], picc_profile,
                   "zset_outputintent");
         if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used for device profile\n");
     } else {
         if (dev_profile->proof_profile == NULL) {
-            /* This means that we should use the OI profile as the proofing 
-               profile.  Note that if someone already has specified a 
+            /* This means that we should use the OI profile as the proofing
+               profile.  Note that if someone already has specified a
                proofing profile it is unclear what they are trying to do
-               with the output intent.  In this case, we will use it 
+               with the output intent.  In this case, we will use it
                just for the source data below */
             dev_profile->proof_profile = picc_profile;
             rc_increment(picc_profile);
@@ -353,17 +359,17 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         switch (picc_profile->data_cs) {
             case gsGRAY:
                 if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used source Gray\n");
-                rc_assign(icc_manager->default_gray, picc_profile, 
+                rc_assign(icc_manager->default_gray, picc_profile,
                           "zset_outputintent");
                 break;
             case gsRGB:
                 if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used source RGB\n");
-                rc_assign(icc_manager->default_rgb, picc_profile, 
+                rc_assign(icc_manager->default_rgb, picc_profile,
                           "zset_outputintent");
                 break;
             case gsCMYK:
                 if_debug0m(gs_debug_flag_icc, imemory, "[icc] OutputIntent used source CMYK\n");
-                rc_assign(icc_manager->default_cmyk, picc_profile, 
+                rc_assign(icc_manager->default_cmyk, picc_profile,
                           "zset_outputintent");
                 break;
             default:
@@ -469,25 +475,23 @@ seticc_lab(i_ctx_t * i_ctx_p, float *white, float *black, float *range_buff)
 {
     int                     code;
     gs_color_space *        pcs;
-    gs_imager_state *       pis = (gs_imager_state *)igs;
     int                     i;
-    static const char *const rfs = LAB_ICC;
 
     /* build the color space object */
-    code = gs_cspace_build_ICC(&pcs, NULL, gs_state_memory(igs));
+    code = gs_cspace_build_ICC(&pcs, NULL, gs_gstate_memory(igs));
     if (code < 0)
         return gs_rethrow(code, "building color space object");
     /* record the current space as the alternative color space */
     /* Get the lab profile.  It may already be set in the icc manager.
        If not then lets populate it.  */
-    if (pis->icc_manager->lab_profile == NULL ) {
+    if (igs->icc_manager->lab_profile == NULL ) {
         /* This can't happen as the profile
            should be initialized during the
            setting of the user params */
         return gs_rethrow(code, "cannot find lab icc profile");
     }
     /* Assign the LAB to LAB profile to this color space */
-    code = gsicc_set_gscs_profile(pcs, pis->icc_manager->lab_profile, gs_state_memory(igs));
+    code = gsicc_set_gscs_profile(pcs, igs->icc_manager->lab_profile, gs_gstate_memory(igs));
     if (code < 0)
         return gs_rethrow(code, "installing the lab profile");
     pcs->cmm_icc_profile_data->Range.ranges[0].rmin = 0.0;
@@ -510,8 +514,7 @@ seticc_cal(i_ctx_t * i_ctx_p, float *white, float *black, float *gamma,
 {
     int                     code;
     gs_color_space *        pcs;
-    gs_imager_state *       pis = (gs_imager_state *)igs;
-    gs_memory_t             *mem = pis->memory;
+    gs_memory_t             *mem = igs->memory;
     int                     i;
     cmm_profile_t           *cal_profile;
 
@@ -531,7 +534,7 @@ seticc_cal(i_ctx_t * i_ctx_p, float *white, float *black, float *gamma,
         cal_profile = gsicc_create_from_cal(white, black, gamma, matrix,
                                             mem->stable_memory, num_colorants);
         if (cal_profile == NULL)
-            return gs_rethrow(-1, "creating the cal profile");
+            return gs_rethrow(gs_error_VMerror, "creating the cal profile failed");
         /* Assign the profile to this color space */
         code = gsicc_set_gscs_profile(pcs, cal_profile, mem->stable_memory);
         /* profile is created with ref count of 1, gsicc_set_gscs_profile()
@@ -574,12 +577,15 @@ znumicc_components(i_ctx_t * i_ctx_p)
         return_error(gs_error_undefined);
     check_read_file(i_ctx_p, s, pstrmval);
 
-    picc_profile = gsicc_profile_new(s, gs_state_memory(igs), NULL, 0);
+    picc_profile = gsicc_profile_new(s, gs_gstate_memory(igs), NULL, 0);
+    if (picc_profile == NULL)
+        return gs_throw(gs_error_VMerror, "Creation of ICC profile failed");
+
     picc_profile->num_comps = ncomps;
     picc_profile->profile_handle =
         gsicc_get_profile_handle_buffer(picc_profile->buffer,
                                         picc_profile->buffer_size,
-                                        gs_state_memory(igs));
+                                        gs_gstate_memory(igs));
     if (picc_profile->profile_handle == NULL) {
         rc_decrement(picc_profile,"znumicc_components");
         make_int(op, expected);
@@ -616,7 +622,7 @@ znumicc_components(i_ctx_t * i_ctx_p)
 
 const op_def    zicc_ll3_op_defs[] = {
     op_def_begin_ll3(),
-    { "1.seticcspace", zseticcspace },    
+    { "1.seticcspace", zseticcspace },
     { "1.set_outputintent", zset_outputintent },
     { "1.numicc_components", znumicc_components },
     op_def_end(0)
