@@ -478,12 +478,13 @@ static void *cups_read(ImageReader *im,
     if (skip_bytes(im->file, 1796-424) == EOF)
         return NULL;
 
-    d = data = Malloc(*width * *height * 4);
+    data = Malloc(*width * *height * 4);
     *span = *width * 4;
     *bpp = 32;
     for (y = *height; y > 0; y--) {
         b = 0;
         c = 0;
+        d = data + (y - 1) * *span;
         for (x = *width; x > 0; x--) {
             b >>= 1;
             if (b == 0) {
@@ -923,8 +924,10 @@ static void pam_header_read(FILE *file,
         } else if (skip_string(file, "MAXVAL")) {
             *maxval = get_pnm_num(file);
         } else if (skip_string(file, "TUPLTYPE")) {
-            if (!skip_string(file, "CMYK")) {
-                fprintf(stderr, "bmpcmp: Only CMYK PAMs!\n");
+            if (skip_string(file, "RGB_TAG")) {
+                /* Treat as CMYK. */
+            } else if (!skip_string(file, "CMYK")) {
+                fprintf(stderr, "bmpcmp: Only CMYK or RGB_ALPHA PAMs!\n");
                 exit(1);
             }
         } else if (skip_string(file, "ENDHDR")) {
